////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/CANopenTypes.cpp $
//
//	project   :  CAN - Controller Area Network
//	
//	purpose   :  CANopen Data Types (CiA Encoding rules)
//	
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CCANopenTypes
//	
//	includes  :  CANopenTypes.h
//	
//	author    :  Uwe Vogt, UV Software
//	
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	CANopen Data Types (CiA Encoding rules)

static const char _id[] = "$Id: CANopenTypes.cpp 47 2009-10-15 19:10:29Z saturn $";

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "CANopenTypes.h"
#include <qobject.h>
#include <qstring.h>
#include <qdatetime.h>

//////////////////////////////////////////////////////////////////////////////
//	prototypes

static int scan_num(const char* lpsz, int* i, int* num);
static int make_dat(int year, int month, int day, int hour, int minute, int second, int milli, SDO_t_DATE *dat);
static int make_tod(int year, int month, int day, int hour, int minute, int second, int milli, SDO_t_TIME_OF_DAY *tod);

//////////////////////////////////////////////////////////////////////////////
//	class CCANopenTypes

bool CCANopenTypes::ascii2unsigned8( const char* string, SDO_t_UNSIGNED8* value )
{
	Q_ASSERT(string);
	Q_ASSERT(value);

	*value = 0;

	int n = 0;
	const char* ptr;

	for(ptr = string; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
				if((char)'0' <= *ptr && *ptr <= (char)'9')
					*value = (*value << 4) + (unsigned char)(*ptr - (char)'0');
				else if((char)'a' <= *ptr && *ptr <= (char)'f')
					*value = (*value << 4) + (unsigned char)(*ptr - (char)'a' + (char)10);
				else if((char)'A' <= *ptr && *ptr <= (char)'F')
					*value = (*value << 4) + (unsigned char)(*ptr - (char)'A' + (char)10);
				else
					return false;
			}
			for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
				;
			if(*ptr != (char)'\0')
				return false;
			if(n > 2)
				return false;
			else
				return true;
		}
		// octal number
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 2 && *value > 037)
	       		n = 4;
			if((char)'0' <= *ptr && *ptr <= (char)'7')
				*value = (*value << 3) + (unsigned char)(*ptr - (char)'0');
			else
				return false;
		}
		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 3)
			return false;
		else
			return true;
	}
	// decimal number
	if(*ptr == (char)'+')
		if(*++ptr == (char)'\0')
			return false;
	for(; *ptr == (char)'0'; ptr++)
		;
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		// overflow?
		if(n == 2 && *value > 25)
       		n = 4;
		if(n == 2 && *value == 25 && *ptr > (char)'5')
       		n = 4;
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*value = (*value * 10) + (unsigned char)(*ptr - (char)'0');
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 3)
		return false;
	else
		return true;
}

bool CCANopenTypes::ascii2integer8( const char* string, SDO_t_INTEGER8* value )
{
	Q_ASSERT(string);
	Q_ASSERT(value);

	*value = 0;

	int n = 0;
	const char* ptr;

	for(ptr = string; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
				if((char)'0' <= *ptr && *ptr <= (char)'9')
					*value = (*value << 4) + (unsigned char)(*ptr - (char)'0');
				else if((char)'a' <= *ptr && *ptr <= (char)'f')
					*value = (*value << 4) + (unsigned char)(*ptr - (char)'a' + (char)10);
				else if((char)'A' <= *ptr && *ptr <= (char)'F')
					*value = (*value << 4) + (unsigned char)(*ptr - (char)'A' + (char)10);
				else
					return false;
			}
			for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
				;
			if(*ptr != (char)'\0')
				return false;
			if(n > 2)
				return false;
			else
				return true;
		}
		// octal number
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 2 && *value > 037)
	       		n = 4;
			if((char)'0' <= *ptr && *ptr <= (char)'7')
				*value = (*value << 3) + (unsigned char)(*ptr - (char)'0');
			else
				return false;
		}
		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 3)
			return false;
		else
			return true;
	}
	// negative decimal number
	if(*ptr == (char)'-') {
		if(*++ptr == (char)'\0')
			return false;
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 2 && *value > 12)
	       		n = 4;
			if(n == 2 && *value == 12 && *ptr > (char)'8')
	       		n = 4;
			if((char)'0' <= *ptr && *ptr <= (char)'9')
				*value = (*value * 10) + (unsigned char)(*ptr - (char)'0');
			else
				return false;
		}
		*value = *value * -1;

		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 3)
			return false;
		else
			return true;
	}
	// positive decimal number
	if(*ptr == (char)'+')
		if(*++ptr == (char)'\0')
			return false;
	for(; *ptr == (char)'0'; ptr++)
		;
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		// overflow?
		if(n == 2 && *value > 12)
       		n = 4;
		if(n == 2 && *value == 12 && *ptr > (char)'7')
       		n = 4;
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*value = (*value * 10) + (unsigned char)(*ptr - (char)'0');
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 3)
		return false;
	else
		return true;
}

bool CCANopenTypes::ascii2unsigned16( const char* string, SDO_t_UNSIGNED16* value )
{
	Q_ASSERT(string);
	Q_ASSERT(value);

	*value = 0;

	int n = 0;
	const char* ptr;

	for(ptr = string; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
				if((char)'0' <= *ptr && *ptr <= (char)'9')
					*value = (*value << 4) + (unsigned short)(*ptr - (char)'0');
				else if((char)'a' <= *ptr && *ptr <= (char)'f')
					*value = (*value << 4) + (unsigned short)(*ptr - (char)'a' + (char)10);
				else if((char)'A' <= *ptr && *ptr <= (char)'F')
					*value = (*value << 4) + (unsigned short)(*ptr - (char)'A' + (char)10);
				else
					return false;
			}
			for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
				;
			if(*ptr != (char)'\0')
				return false;
			if(n > 4)
				return false;
			else
				return true;
		}
		// octal number
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 5 && *value > 017777)
	       		n = 7;
			if((char)'0' <= *ptr && *ptr <= (char)'7')
				*value = (*value << 3) + (unsigned short)(*ptr - (char)'0');
			else
				return false;
		}
		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 6)
			return false;
		else
			return true;
	}
	// decimal number
	if(*ptr == (char)'+')
		if(*++ptr == (char)'\0')
			return false;
	for(; *ptr == (char)'0'; ptr++)
		;
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		// overflow?
		if(n == 4 && *value > 6553)
       		n = 6;
		if(n == 4 && *value == 6553 && *ptr > (char)'5')
       		n = 6;
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*value = (*value * 10) + (unsigned short)(*ptr - (char)'0');
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 5)
		return false;
	else
		return true;
}

bool CCANopenTypes::ascii2integer16( const char* string, SDO_t_INTEGER16* value )
{
	Q_ASSERT(string);
	Q_ASSERT(value);

	*value = 0;

	int n = 0;
	const char* ptr;

	for(ptr = string; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
				if((char)'0' <= *ptr && *ptr <= (char)'9')
					*value = (*value << 4) + (unsigned short)(*ptr - (char)'0');
				else if((char)'a' <= *ptr && *ptr <= (char)'f')
					*value = (*value << 4) + (unsigned short)(*ptr - (char)'a' + (char)10);
				else if((char)'A' <= *ptr && *ptr <= (char)'F')
					*value = (*value << 4) + (unsigned short)(*ptr - (char)'A' + (char)10);
				else
					return false;
			}
			for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
				;
			if(*ptr != (char)'\0')
				return false;
			if(n > 4)
				return false;
			else
				return true;
		}
		// octal number
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 5 && *value > 017777)
	       		n = 7;
			if((char)'0' <= *ptr && *ptr <= (char)'7')
				*value = (*value << 3) + (unsigned short)(*ptr - (char)'0');
			else
				return false;
		}
		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 6)
			return false;
		else
			return true;
	}
	// negative decimal number
	if(*ptr == (char)'-') {
		if(*++ptr == (char)'\0')
			return false;
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 4 && *value > 3276)
	       		n = 6;
			if(n == 4 && *value == 3276 && *ptr > (char)'8')
	       		n = 6;
			if((char)'0' <= *ptr && *ptr <= (char)'9')
				*value = (*value * 10) + (unsigned short)(*ptr - (char)'0');
			else
				return false;
		}
		*value = *value * -1;

		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 5)
			return false;
		else
			return true;
	}
	// positive decimal number
	if(*ptr == (char)'+')
		if(*++ptr == (char)'\0')
			return false;
	for(; *ptr == (char)'0'; ptr++)
		;
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		// overflow?
		if(n == 4 && *value > 3276)
       		n = 6;
		if(n == 4 && *value == 3276 && *ptr > (char)'7')
       		n = 6;
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*value = (*value * 10) + (unsigned short)(*ptr - (char)'0');
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 5)
		return false;
	else
		return true;
}

bool CCANopenTypes::ascii2unsigned32( const char* string, SDO_t_UNSIGNED32* value )
{
	Q_ASSERT(string);
	Q_ASSERT(value);

	*value = 0;

	int n = 0;
	const char* ptr;

	for(ptr = string; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
				if((char)'0' <= *ptr && *ptr <= (char)'9')
					*value = (*value << 4) + (unsigned long)(*ptr - (char)'0');
				else if((char)'a' <= *ptr && *ptr <= (char)'f')
					*value = (*value << 4) + (unsigned long)(*ptr - (char)'a' + (char)10);
				else if((char)'A' <= *ptr && *ptr <= (char)'F')
					*value = (*value << 4) + (unsigned long)(*ptr - (char)'A' + (char)10);
				else
					return false;
			}
			for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
				;
			if(*ptr != (char)'\0')
				return false;
			if(n > 8)
				return false;
			else
				return true;
		}
		// octal number
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			if(n == 10 && *value > 03777777777UL)
	       		n = 12;
			if((char)'0' <= *ptr && *ptr <= (char)'7')
				*value = (*value << 3) + (unsigned long)(*ptr - (char)'0');
			else
				return false;
		}
		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 11)
			return false;
		else
			return true;
	}
	// decimal number
	if(*ptr == (char)'+')
		if(*++ptr == (char)'\0')
			return false;
	for(; *ptr == (char)'0'; ptr++)
		;
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		// overflow?
		if(n == 9 && *value > 429496729UL)
       		n = 11;
		if(n == 9 && *value == 429496729UL && *ptr > (char)'5')
       		n = 11;
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*value = (*value * 10UL) + (unsigned long)(*ptr - (char)'0');
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 10)
		return false;
	else
		return true;
}

bool CCANopenTypes::ascii2integer32( const char* string, SDO_t_INTEGER32* value )
{
	Q_ASSERT(string);
	Q_ASSERT(value);

	*value = 0;

	int n = 0;
	const char* ptr;

	for(ptr = string; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
				if((char)'0' <= *ptr && *ptr <= (char)'9')
					*value = (*value << 4) + (unsigned long)(*ptr - (char)'0');
				else if((char)'a' <= *ptr && *ptr <= (char)'f')
					*value = (*value << 4) + (unsigned long)(*ptr - (char)'a' + (char)10);
				else if((char)'A' <= *ptr && *ptr <= (char)'F')
					*value = (*value << 4) + (unsigned long)(*ptr - (char)'A' + (char)10);
				else
					return false;
			}
			for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
				;
			if(*ptr != (char)'\0')
				return false;
			if(n > 8)
				return false;
			else
				return true;
		}
		// octal number
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			if(n == 10 && *value > 03777777777L)
	       		n = 12;
			if((char)'0' <= *ptr && *ptr <= (char)'7')
				*value = (*value << 3) + (unsigned long)(*ptr - (char)'0');
			else
				return false;
		}
		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 11)
			return false;
		else
			return true;
	}
	// negative decimal number
	if(*ptr == (char)'-') {
		if(*++ptr == (char)'\0')
			return false;
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 9 && *value > 214748364L)
	       		n = 11;
			if(n == 9 && *value == 214748364L && *ptr > (char)'8')
	       		n = 11;
			if((char)'0' <= *ptr && *ptr <= (char)'9')
				*value = (*value * 10UL) + (unsigned long)(*ptr - (char)'0');
			else
				return false;
		}
		*value = *value * -1L;

		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 10)
			return false;
		else
			return true;
	}
	// positive decimal number
	if(*ptr == (char)'+')
		if(*++ptr == (char)'\0')
			return false;
	for(; *ptr == (char)'0'; ptr++)
		;
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		// overflow?
		if(n == 9 && *value > 214748364L)
       		n = 11;
		if(n == 9 && *value == 214748364L && *ptr > (char)'7')
       		n = 11;
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*value = (*value * 10UL) + (unsigned long)(*ptr - (char)'0');
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 10)
		return false;
	else
		return true;
}

bool CCANopenTypes::ascii2timeOfDay( const char* string, SDO_t_TIME_OF_DAY* value )
{
	Q_ASSERT(value);
	Q_ASSERT(string);

	int year = QDate::currentDate().year();
	int month = QDate::currentDate().month();
	int day = QDate::currentDate().day();
	int hour = QTime::currentTime().hour();
	int minute = QTime::currentTime().minute();
	int second = QTime::currentTime().second();
	int milli = 0;
	int num, i = 0;

	while(string[i] == ' ') i++;
	if(string[i] == '\0')
	{
		return make_tod(year, month, day, hour, minute, second, milli, value);
	}
	switch(scan_num(string, &i, &num))
	{
	// Format: dd[.[mm[.[[yy]yy[ [hh[:[mm[:[ss[.[nnn]]]]]]]]]]]]
	case '\0':
		day = num;
		return make_tod(year, month, day, hour, minute, second, milli, value);
	case '.':
		day = num;
		if(string[i] == '\0')
		{
			return make_tod(year, month, day, hour, minute, second, milli, value);
		}
		switch(scan_num(string, &i, &num))
		{
		case '\0':
			month = num;
			return make_tod(year, month, day, hour, minute, second, milli, value);
		case '.':
			month = num;
			if(string[i] == '\0')
			{
				return make_tod(year, month, day, hour, minute, second, milli, value);
			}
			switch(scan_num(string, &i, &num))
			{
			case '\0':
				year = num;
				return make_tod(year, month, day, hour, minute, second, milli, value);
			case ' ':
				year = num;
				while(string[i] == ' ') i++;
				if(string[i] == '\0')
				{
					return make_tod(year, month, day, hour, minute, second, milli, value);
				}
				switch(scan_num(string, &i, &num))
				{
				case '\0':
					hour = num;
					return make_tod(year, month, day, hour, minute, second, milli, value);
				case ':':
					hour = num;
					if(string[i] == '\0')
					{
						return make_tod(year, month, day, hour, minute, second, milli, value);
					}
					switch(scan_num(string, &i, &num))
					{
					case '\0':
						minute = num;
						return make_tod(year, month, day, hour, minute, second, milli, value);
					case ':':
						minute = num;
						if(string[i] == '\0')
						{
							return make_tod(year, month, day, hour, minute, second, milli, value);
						}
						switch(scan_num(string, &i, &num))
						{
						case '\0':
							second = num;
							return make_tod(year, month, day, hour, minute, second, milli, value);
						case '.':
							second = num;
							if(string[i] == '\0')
							{
								return make_tod(year, month, day, hour, minute, second, milli, value);
							}
							switch(scan_num(string, &i, &num))
							{
							case '\0':
								milli = num;
								return make_tod(year, month, day, hour, minute, second, milli, value);
							case ' ':
								milli = num;
								while(string[i] == ' ') i++;
								if(string[i] == '\0')
								{
									return make_tod(year, month, day, hour, minute, second, milli, value);
								}
								return false;
							default:
								return false;
							}
						default:
							return false;
						}
					default:
						return false;
					}
				default:
					return false;
				}
			default:
				return false;
			}
		default:
			return false;
		}
	// Format: mm/[dd[/[yy]yy[ [hh[:[mm[:[ss[.[nnn]]]]]]]]]]]
	case '/':
		month = num;
		if(string[i] == '\0')
		{
			return make_tod(year, month, day, hour, minute, second, milli, value);
		}
		switch(scan_num(string, &i, &num))
		{
		case '\0':
			day = num;
			return make_tod(year, month, day, hour, minute, second, milli, value);
		case '/':
			day = num;
			if(string[i] == '\0')
			{
				return make_tod(year, month, day, hour, minute, second, milli, value);
			}
			switch(scan_num(string, &i, &num))
			{
			case '\0':
				year = num;
				return make_tod(year, month, day, hour, minute, second, milli, value);
			case ' ':
				year = num;
				while(string[i] == ' ') i++;
				if(string[i] == '\0')
				{
					return make_tod(year, month, day, hour, minute, second, milli, value);
				}
				switch(scan_num(string, &i, &num))
				{
				case '\0':
					hour = num;
					return make_tod(year, month, day, hour, minute, second, milli, value);
				case ':':
					hour = num;
					if(string[i] == '\0')
					{
						return make_tod(year, month, day, hour, minute, second, milli, value);
					}
					switch(scan_num(string, &i, &num))
					{
					case '\0':
						minute = num;
						return make_tod(year, month, day, hour, minute, second, milli, value);
					case ':':
						minute = num;
						if(string[i] == '\0')
						{
							return make_tod(year, month, day, hour, minute, second, milli, value);
						}
						switch(scan_num(string, &i, &num))
						{
						case '\0':
							second = num;
							return make_tod(year, month, day, hour, minute, second, milli, value);
						case '.':
							second = num;
							if(string[i] == '\0')
							{
								return make_tod(year, month, day, hour, minute, second, milli, value);
							}
							switch(scan_num(string, &i, &num))
							{
							case '\0':
								milli = num;
								return make_tod(year, month, day, hour, minute, second, milli, value);
							case ' ':
								milli = num;
								while(string[i] == ' ') i++;
								if(string[i] == '\0')
								{
									return make_tod(year, month, day, hour, minute, second, milli, value);
								}
								return false;
							default:
								return false;
							}
						default:
							return false;
						}
					default:
						return false;
					}
				default:
					return false;
				}
			default:
				return false;
			}
		default:
			return false;
		}
	// Format: [yy]yy-[mm[-dd[ [hh[:[mm[:[ss[.[nnn]]]]]]]]]]]
	case '-':
		year = num;
		if(string[i] == '\0')
		{
			return make_tod(year, month, day, hour, minute, second, milli, value);
		}
		switch(scan_num(string, &i, &num))
		{
		case '\0':
			month = num;
			return make_tod(year, month, day, hour, minute, second, milli, value);
		case '-':
			month = num;
			if(string[i] == '\0')
			{
				return make_tod(year, month, day, hour, minute, second, milli, value);
			}
			switch(scan_num(string, &i, &num))
			{
			case '\0':
				day = num;
				return make_tod(year, month, day, hour, minute, second, milli, value);
			case ' ':
				day = num;
				while(string[i] == ' ') i++;
				if(string[i] == '\0')
				{
					return make_tod(year, month, day, hour, minute, second, milli, value);
				}
				switch(scan_num(string, &i, &num))
				{
				case '\0':
					hour = num;
					return make_tod(year, month, day, hour, minute, second, milli, value);
				case ':':
					hour = num;
					if(string[i] == '\0')
					{
						return make_tod(year, month, day, hour, minute, second, milli, value);
					}
					switch(scan_num(string, &i, &num))
					{
					case '\0':
						minute = num;
						return make_tod(year, month, day, hour, minute, second, milli, value);
					case ':':
						minute = num;
						if(string[i] == '\0')
						{
							return make_tod(year, month, day, hour, minute, second, milli, value);
						}
						switch(scan_num(string, &i, &num))
						{
						case '\0':
							second = num;
							return make_tod(year, month, day, hour, minute, second, milli, value);
						case '.':
							second = num;
							if(string[i] == '\0')
							{
								return make_tod(year, month, day, hour, minute, second, milli, value);
							}
							switch(scan_num(string, &i, &num))
							{
							case '\0':
								milli = num;
								return make_tod(year, month, day, hour, minute, second, milli, value);
							case ' ':
								milli = num;
								while(string[i] == ' ') i++;
								if(string[i] == '\0')
								{
									return make_tod(year, month, day, hour, minute, second, milli, value);
								}
								return false;
							default:
								return false;
							}
						default:
							return false;
						}
					default:
						return false;
					}
				default:
					return false;
				}
			default:
				return false;
			}
		default:
			return false;
		}
	default:
		return false;
	}
}

bool CCANopenTypes::ascii2date( const char* string, SDO_t_DATE* value )
{
	Q_ASSERT(value);
	Q_ASSERT(string);

	int year = QDate::currentDate().year();
	int month = QDate::currentDate().month();
	int day = QDate::currentDate().day();
	int hour = QTime::currentTime().hour();
	int minute = QTime::currentTime().minute();
	int second = QTime::currentTime().second();
	int milli = 0;
	int num, i = 0;

	while(string[i] == ' ') i++;
	if(string[i] == '\0')
	{
		return make_dat(year, month, day, hour, minute, second, milli, value);
	}
	switch(scan_num(string, &i, &num))
	{
	// Format: dd[.[mm[.[[yy]yy[ [hh[:[mm[:[ss[.[nnn]]]]]]]]]]]]
	case '\0':
		day = num;
		return make_dat(year, month, day, hour, minute, second, milli, value);
	case '.':
		day = num;
		if(string[i] == '\0')
		{
			return make_dat(year, month, day, hour, minute, second, milli, value);
		}
		switch(scan_num(string, &i, &num))
		{
		case '\0':
			month = num;
			return make_dat(year, month, day, hour, minute, second, milli, value);
		case '.':
			month = num;
			if(string[i] == '\0')
			{
				return make_dat(year, month, day, hour, minute, second, milli, value);
			}
			switch(scan_num(string, &i, &num))
			{
			case '\0':
				year = num;
				return make_dat(year, month, day, hour, minute, second, milli, value);
			case ' ':
				year = num;
				while(string[i] == ' ') i++;
				if(string[i] == '\0')
				{
					return make_dat(year, month, day, hour, minute, second, milli, value);
				}
				switch(scan_num(string, &i, &num))
				{
				case '\0':
					hour = num;
					return make_dat(year, month, day, hour, minute, second, milli, value);
				case ':':
					hour = num;
					if(string[i] == '\0')
					{
						return make_dat(year, month, day, hour, minute, second, milli, value);
					}
					switch(scan_num(string, &i, &num))
					{
					case '\0':
						minute = num;
						return make_dat(year, month, day, hour, minute, second, milli, value);
					case ':':
						minute = num;
						if(string[i] == '\0')
						{
							return make_dat(year, month, day, hour, minute, second, milli, value);
						}
						switch(scan_num(string, &i, &num))
						{
						case '\0':
							second = num;
							return make_dat(year, month, day, hour, minute, second, milli, value);
						case '.':
							second = num;
							if(string[i] == '\0')
							{
								return make_dat(year, month, day, hour, minute, second, milli, value);
							}
							switch(scan_num(string, &i, &num))
							{
							case '\0':
								milli = num;
								return make_dat(year, month, day, hour, minute, second, milli, value);
							case ' ':
								milli = num;
								while(string[i] == ' ') i++;
								if(string[i] == '\0')
								{
									return make_dat(year, month, day, hour, minute, second, milli, value);
								}
								return false;
							default:
								return false;
							}
						default:
							return false;
						}
					default:
						return false;
					}
				default:
					return false;
				}
			default:
				return false;
			}
		default:
			return false;
		}
	// Format: mm/[dd[/[yy]yy[ [hh[:[mm[:[ss[.[nnn]]]]]]]]]]]
	case '/':
		month = num;
		if(string[i] == '\0')
		{
			return make_dat(year, month, day, hour, minute, second, milli, value);
		}
		switch(scan_num(string, &i, &num))
		{
		case '\0':
			day = num;
			return make_dat(year, month, day, hour, minute, second, milli, value);
		case '/':
			day = num;
			if(string[i] == '\0')
			{
				return make_dat(year, month, day, hour, minute, second, milli, value);
			}
			switch(scan_num(string, &i, &num))
			{
			case '\0':
				year = num;
				return make_dat(year, month, day, hour, minute, second, milli, value);
			case ' ':
				year = num;
				while(string[i] == ' ') i++;
				if(string[i] == '\0')
				{
					return make_dat(year, month, day, hour, minute, second, milli, value);
				}
				switch(scan_num(string, &i, &num))
				{
				case '\0':
					hour = num;
					return make_dat(year, month, day, hour, minute, second, milli, value);
				case ':':
					hour = num;
					if(string[i] == '\0')
					{
						return make_dat(year, month, day, hour, minute, second, milli, value);
					}
					switch(scan_num(string, &i, &num))
					{
					case '\0':
						minute = num;
						return make_dat(year, month, day, hour, minute, second, milli, value);
					case ':':
						minute = num;
						if(string[i] == '\0')
						{
							return make_dat(year, month, day, hour, minute, second, milli, value);
						}
						switch(scan_num(string, &i, &num))
						{
						case '\0':
							second = num;
							return make_dat(year, month, day, hour, minute, second, milli, value);
						case '.':
							second = num;
							if(string[i] == '\0')
							{
								return make_dat(year, month, day, hour, minute, second, milli, value);
							}
							switch(scan_num(string, &i, &num))
							{
							case '\0':
								milli = num;
								return make_dat(year, month, day, hour, minute, second, milli, value);
							case ' ':
								milli = num;
								while(string[i] == ' ') i++;
								if(string[i] == '\0')
								{
									return make_dat(year, month, day, hour, minute, second, milli, value);
								}
								return false;
							default:
								return false;
							}
						default:
							return false;
						}
					default:
						return false;
					}
				default:
					return false;
				}
			default:
				return false;
			}
		default:
			return false;
		}
	// Format: [yy]yy-[mm[-dd[ [hh[:[mm[:[ss[.[nnn]]]]]]]]]]]
	case '-':
		year = num;
		if(string[i] == '\0')
		{
			return make_dat(year, month, day, hour, minute, second, milli, value);
		}
		switch(scan_num(string, &i, &num))
		{
		case '\0':
			month = num;
			return make_dat(year, month, day, hour, minute, second, milli, value);
		case '-':
			month = num;
			if(string[i] == '\0')
			{
				return make_dat(year, month, day, hour, minute, second, milli, value);
			}
			switch(scan_num(string, &i, &num))
			{
			case '\0':
				day = num;
				return make_dat(year, month, day, hour, minute, second, milli, value);
			case ' ':
				day = num;
				while(string[i] == ' ') i++;
				if(string[i] == '\0')
				{
					return make_dat(year, month, day, hour, minute, second, milli, value);
				}
				switch(scan_num(string, &i, &num))
				{
				case '\0':
					hour = num;
					return make_dat(year, month, day, hour, minute, second, milli, value);
				case ':':
					hour = num;
					if(string[i] == '\0')
					{
						return make_dat(year, month, day, hour, minute, second, milli, value);
					}
					switch(scan_num(string, &i, &num))
					{
					case '\0':
						minute = num;
						return make_dat(year, month, day, hour, minute, second, milli, value);
					case ':':
						minute = num;
						if(string[i] == '\0')
						{
							return make_dat(year, month, day, hour, minute, second, milli, value);
						}
						switch(scan_num(string, &i, &num))
						{
						case '\0':
							second = num;
							return make_dat(year, month, day, hour, minute, second, milli, value);
						case '.':
							second = num;
							if(string[i] == '\0')
							{
								return make_dat(year, month, day, hour, minute, second, milli, value);
							}
							switch(scan_num(string, &i, &num))
							{
							case '\0':
								milli = num;
								return make_dat(year, month, day, hour, minute, second, milli, value);
							case ' ':
								milli = num;
								while(string[i] == ' ') i++;
								if(string[i] == '\0')
								{
									return make_dat(year, month, day, hour, minute, second, milli, value);
								}
								return false;
							default:
								return false;
							}
						default:
							return false;
						}
					default:
						return false;
					}
				default:
					return false;
				}
			default:
				return false;
			}
		default:
			return false;
		}
	default:
		return false;
	}
}

bool CCANopenTypes::timeOfDay2ascii( SDO_t_TIME_OF_DAY timeOfDay, QString& ascii )
{
	unsigned long t_time = (((unsigned long)timeOfDay.member.days + 5113UL) * 86400UL)
		        		 +  ((unsigned long)timeOfDay.member.msec / 1000UL);

	if((unsigned long)t_time > 2145916800UL)	// less than 01/01/2038!
					  t_time = (time_t)0;

	QDateTime q_time;
	q_time.setTime_t( t_time, Qt::LocalTime );
#if 1
	ascii = q_time.toString( "yyyy-MM-dd hh:mm:ss.zzz" );
#else
	ascii = q_time.toString( Qt::ISODate );
#endif

	return true;
}

bool CCANopenTypes::date2ascii( SDO_t_DATE date, QString& ascii )
{
	if(!date.member.day_of_week && !date.member.month &&
	   !date.member.day_of_month && !date.member.year)
	{
		date.member.day_of_week = 4;
		date.member.month = 1;
		date.member.day_of_month = 1;
		date.member.year = 70;
	}
	QDateTime q_time( QDate( date.member.year  + (date.member.year < 70? 2000 : 1900),
							 date.member.month,
							 date.member.day_of_month),
					  QTime( date.member.hour,
							 date.member.minutes,
							 date.member.milliseconds / 1000 ,
							 date.member.milliseconds % 1000 ) );
#if 1
	ascii = q_time.toString( "yyyy-MM-dd hh:mm:ss.zzz" );
#else
	ascii = q_time.toString( Qt::ISODate );
#endif

	return true;
}

//////////////////////////////////////////////////////////////////////////////
//	mother´s little helpers

static int scan_num(const char* lpsz, int* i, int* num)
{
	int tmp = 0;

	if(lpsz[(*i)] == '\0') {
		*num = tmp;
		return lpsz[(*i)++];
	}
	while(isdigit(lpsz[(*i)])) {
		tmp = (tmp * 10) + (lpsz[(*i)++] - '0');
	}
	*num = tmp;
	return lpsz[(*i)++];
}

static int make_dat(int year, int month, int day,
					int hour, int minute, int second, int milli, SDO_t_DATE *dat)
{
	if(year < 100) {
		if(year >= 70)
			year += 1900;
		else
			year += 2000;
	}
	if((year < 1970) || (2038 <= year)) return FALSE;
	if((month < 1) || (12 < month)) return FALSE;
	if((day < 1) || (31 < day)) return FALSE;
	if((hour < 0) || (23 < hour)) return FALSE;
	if((minute < 0) || (59 < minute)) return FALSE;
	if((second < 0) || (59 < second)) return FALSE;
	if((milli < 0) || (999 < milli)) return FALSE;

	unsigned tmp;
	tmp = ((unsigned)second * 1000) + (unsigned)milli;

	if(dat) {
	   dat->data[0] = (unsigned char)(tmp &0x00FF) >> 0;
	   dat->data[1] = (unsigned char)(tmp &0xFF00) >> 8;
	   dat->data[2] = (unsigned char)minute & 0x3F;
	   dat->data[3] = (unsigned char)hour & 0x1F;
	   dat->data[4] = (unsigned char)day & 0x1F;
	   dat->data[5] = (unsigned char)month & 0x3F;
	   dat->data[6] = (unsigned char)(year % 100) & 0x7F;
	}
	return TRUE;
}

static int make_tod(int year, int month, int day,
					int hour, int minute, int second, int milli, SDO_t_TIME_OF_DAY *tod)
{
	if(year < 100) {
		if(year >= 84)
			year += 1900;
		else
			year += 2000;
	}
	if((year < 1984) || 2038 <= year) return FALSE;
	if((month < 1) || (12 < month)) return FALSE;
	if((day < 1) || (31 < day)) return FALSE;
	if((hour < 0) || (23 < hour)) return FALSE;
	if((minute < 0) || (59 < minute)) return FALSE;
	if((second < 0) || (59 < second)) return FALSE;
	if((milli < 0) || (999 < milli)) return FALSE;

	struct tm tmp;
	memset(&tmp, 0, sizeof(struct tm));
	tmp.tm_year = year - 1900;
	tmp.tm_mon  = month - 1;
	tmp.tm_mday = day;
	tmp.tm_hour = hour;
	tmp.tm_min  = minute;
	tmp.tm_sec  = second;
	time_t t_time = mktime(&tmp);

	if(tod) {
	   tod->member.days  = (unsigned short)(t_time / 86400UL) - 5113;
	   tod->member.msec  = (unsigned long )(t_time % 86400UL) * 1000UL;
	   tod->member.msec += (unsigned long )(milli);
	}
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////////
//	revision control

const char* CCANopenTypes::version()
{
	return (const char*)_id;
}

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
