//Added by qt3to4:
#include <Q3GridLayout>
#include <Q3HBoxLayout>
#include <Q3VBoxLayout>
////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/InitInterfaceDialog.cpp $
//
//	project   :  COPbrowser - CANopen Object Browser
//	
//	purpose   :  InitInterfaceDialog - Dialog
//	
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CInitInterfaceDialog
//	
//	includes  :  InitInterfaceDialog.h, <includes>
//	
//	author    :  Uwe Vogt, UV Software
//	
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	<description>

static const char _id[] = "$Id: InitInterfaceDialog.cpp 56 2009-11-05 18:48:27Z saturn $";

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "InitInterfaceDialog.h"
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <q3whatsthis.h>
#include <qmessagebox.h>
#include <qsettings.h>
#include "COPbrowser.h"

//////////////////////////////////////////////////////////////////////////////
//	class CInitInterfaceDialog

CInitInterfaceDialog::CInitInterfaceDialog( QWidget* parent, const char * name )
	: QDialog( parent, name, FALSE, 0 )
{
	if ( !name )
		setName( "Initialize Interface dialog" );
	Q3VBoxLayout* dialogLayout = new Q3VBoxLayout( this, 11, 6, "Initialize CAN Interface layout");

	QLabel* interfaceLabel = new QLabel( "&Interface:", this, "Interface text label" );
	m_interfaceComboBox = new QComboBox( FALSE, this, "Interface combo box" );
	m_interfaceComboBox->setEditable( TRUE );
	m_interfaceComboBox->setDuplicatesEnabled( FALSE );
	m_interfaceComboBox->setInsertionPolicy( QComboBox::AtTop );
    interfaceLabel->setBuddy( m_interfaceComboBox );

	QLabel* baudrateLabel = new QLabel( "&Baud rate:", this, "Baud rate text label" );
	m_baudrateComboBox = new QComboBox( FALSE, this, "Baud rate combo box" );
	baudrateLabel->setBuddy( m_baudrateComboBox );
	QLabel* baudLabel = new QLabel( "Kbps (CiA Bit-timing)", this, "Kbps text label" );

	QLabel* masterIdLabel = new QLabel( "&Node-Id.:", this, "Node-Id. text label" );
	m_masterIdSpinBox = new QSpinBox( this, "nodeIdSpinBox" );
	m_masterIdSpinBox->setMaxValue( 127 );
	m_masterIdSpinBox->setMinValue( 0 );
	m_masterIdSpinBox->setValue( 0 );
	masterIdLabel->setBuddy( m_masterIdSpinBox );
	QLabel* masterLabel = new QLabel( "for CANopen Master", this, "Master text label" );

	m_okPushButton = new QPushButton( this, "OK push button" );
	m_okPushButton->setText( "&OK" );
	m_okPushButton->setAccel( QKeySequence( "Alt+O" ) );
	m_okPushButton->setDefault( TRUE );

	m_cancelPushButton = new QPushButton( this, "Cancel push button" );
	m_cancelPushButton->setText( "&Cancel" );
	m_cancelPushButton->setAccel( QKeySequence( "Alt+C" ) );

	Q3GridLayout* localUpperLayout = new Q3GridLayout( 0, 1, 1, 0, 6, "local upper layout");
    localUpperLayout->addWidget( interfaceLabel, 0, 0 );
    localUpperLayout->addMultiCellWidget( m_interfaceComboBox, 0, 0, 1, 4 );
    localUpperLayout->addWidget( baudrateLabel, 1, 0 );
    localUpperLayout->addWidget( m_baudrateComboBox, 1, 1 );
    localUpperLayout->addWidget( baudLabel, 1, 2 );
    localUpperLayout->addWidget( masterIdLabel, 2, 0 );
    localUpperLayout->addWidget( m_masterIdSpinBox, 2, 1 );
    localUpperLayout->addMultiCellWidget( masterLabel, 2, 2, 2, 3 );
    QSpacerItem* spacer1 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    localUpperLayout->addItem( spacer1, 2, 4 );

    Q3HBoxLayout* localLowerLayout = new Q3HBoxLayout( 0, 0, 6, "local lower layout");
    QSpacerItem* spacer2 = new QSpacerItem( 91, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    localLowerLayout->addItem( spacer2 );
    localLowerLayout->addWidget( m_okPushButton );
    localLowerLayout->addWidget( m_cancelPushButton );

    dialogLayout->addLayout( localUpperLayout );
    QSpacerItem* spacer3 = new QSpacerItem( 20, 16, QSizePolicy::Minimum, QSizePolicy::Expanding );
    dialogLayout->addItem( spacer3 );
    dialogLayout->addLayout( localLowerLayout );

	// tool tip
	QToolTip::add( m_interfaceComboBox, INIT_INTERFACE_INTERFACE_STATUSTIP );
	QToolTip::add( m_baudrateComboBox, INIT_INTERFACE_BAUDRATE_STATUSTIP );
	QToolTip::add( m_masterIdSpinBox, INIT_INTERFACE_MASTER_ID_STATUSTIP );
	QToolTip::add( m_okPushButton, INIT_INTERFACE_ACCEPT_STATUSTIP );
	QToolTip::add( m_cancelPushButton, INIT_INTERFACE_CANCEL_STATUSTIP );

	// context help
	Q3WhatsThis::add( interfaceLabel, INIT_INTERFACE_INTERFACE_WHATsTHIS );
	Q3WhatsThis::add( baudrateLabel, INIT_INTERFACE_BAUDRATE_WHATsTHIS );
	Q3WhatsThis::add( baudLabel, INIT_INTERFACE_BAUDRATE_WHATsTHIS );
	Q3WhatsThis::add( masterIdLabel, INIT_INTERFACE_MASTER_ID_WHATsTHIS );
	Q3WhatsThis::add( masterLabel, INIT_INTERFACE_MASTER_ID_WHATsTHIS );
	Q3WhatsThis::add( m_interfaceComboBox, INIT_INTERFACE_INTERFACE_WHATsTHIS );
	Q3WhatsThis::add( m_baudrateComboBox, INIT_INTERFACE_BAUDRATE_WHATsTHIS );
	Q3WhatsThis::add( m_masterIdSpinBox, INIT_INTERFACE_MASTER_ID_WHATsTHIS );
	Q3WhatsThis::add( m_okPushButton, INIT_INTERFACE_ACCEPT_WHATsTHIS );
	Q3WhatsThis::add( m_cancelPushButton, INIT_INTERFACE_CANCEL_WHATsTHIS );

    // signal/slot connections
    connect( m_okPushButton, SIGNAL( clicked() ), this, SLOT( acceptInitialize() ) );
    connect( m_cancelPushButton, SIGNAL( clicked() ), this, SLOT( reject() ) );

    // populate the baud rate combo box
    m_baudrateComboBox->insertItem( "1000" );
    m_baudrateComboBox->insertItem( "800" );
    m_baudrateComboBox->insertItem( "500" );
    m_baudrateComboBox->insertItem( "250" );
    m_baudrateComboBox->insertItem( "125" );
    m_baudrateComboBox->insertItem( "100" );
    m_baudrateComboBox->insertItem( "50" );
    m_baudrateComboBox->insertItem( "20" );
    m_baudrateComboBox->insertItem( "10" );
    m_baudrateComboBox->setCurrentItem( 3 );

    // load user settings
    loadSettings();

    //@ToDo: realize it!
	m_baudrateComboBox->setEnabled( FALSE );
	baudrateLabel->setEnabled( FALSE );
	baudLabel->setEnabled( FALSE );
	m_masterIdSpinBox->setEnabled( FALSE );
	masterIdLabel->setEnabled( FALSE );
	masterLabel->setEnabled( FALSE );

	// resize the dialog box
    resize( QSize(330, 170).expandedTo(minimumSizeHint()) );
    //clearWState( WState_Polished );
    setCaption( "Initialize CAN Interface" );
}

CInitInterfaceDialog::~CInitInterfaceDialog()
{
}

const char* CInitInterfaceDialog::getInterface()
{
	QString interface = m_interfaceComboBox->currentText();
	for( int i = 0; i < m_interfaceComboBox->count(); i++ ) {
		if( m_interfaceComboBox->text( i ) == interface ) {
			m_interfaceComboBox->removeItem( i );
			break;
		}
	}
	m_interfaceComboBox->insertItem( interface, 0 );
	return interface;
}

int CInitInterfaceDialog::getBaudrate()
{
	return m_baudrateComboBox->currentItem();
}

void CInitInterfaceDialog::loadSettings()
{
	QSettings settings;
	settings.insertSearchPath( QSettings::Windows,  QString( "/%1" ).arg( WINDOWS_REGISTRY ) );
	for( int i = 0; i < 10; i++ ) {
		QString interfaceEntry = settings.readEntry( QString( "/%1/%2/%3%4" )
				.arg( APPLICATION_KEY )
				.arg( INTERFACES_KEY )
				.arg( INTERFACEi_KEY )
				.arg( i+1 ) );
		if( !interfaceEntry.isNull() && !interfaceEntry.isEmpty() )
			m_interfaceComboBox->insertItem( interfaceEntry );
	}
}

void CInitInterfaceDialog::saveSettings()
{
	QSettings settings;
	settings.insertSearchPath( QSettings::Windows,  QString( "/%1" ).arg( WINDOWS_REGISTRY ) );
	for( int i = 0; i < m_interfaceComboBox->count() && i < 10; i++ ) {
		settings.writeEntry( QString( "/%1/%2/%3%4" )
				.arg( APPLICATION_KEY )
				.arg( INTERFACES_KEY )
				.arg( INTERFACEi_KEY )
				.arg( i+1 ),  m_interfaceComboBox->text( i ) );
	}
}

void CInitInterfaceDialog::acceptInitialize()
{
	if( !m_interfaceComboBox->currentText().isNull() &&
		!m_interfaceComboBox->currentText().isEmpty() )
	{
		done( QDialog::Accepted );
	}
	else
	{
		QString err = QString( "No interface given!\n\nPlease retry." );
		QMessageBox::warning( this, PROGRAM_STRING" - Initialize Interface", err);
	}
}

//////////////////////////////////////////////////////////////////////////////
//	revision control

const char* CInitInterfaceDialog::version()
{
	return (const char*)_id;
}

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
