//Added by qt3to4:
#include <Q3Frame>
#include <q3mimefactory.h>
#include <QCloseEvent>
////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/MainFrame.cpp $
//
//	project   :  COPbrowser - CANopen Object Browser
//
//	purpose   :  MainFrame - Main Window of the Application
//
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CMainFrame
//
//	includes  :  COPbrowser.h, cop/cop_api.h
//
//	author    :  Uwe Vogt, UV Software
//
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	MainFrame - Main Window of the Application

static const char _id[] = "$Id: MainFrame.cpp 60 2009-11-07 21:47:27Z saturn $";

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "COPbrowser.h"
#include "MainFrame.h"
#include <qapplication.h>
#include <qmessagebox.h>
#include <qsettings.h>
#include <qworkspace.h>
#include <qmenubar.h>
#include <q3popupmenu.h>
#include <q3toolbar.h>
#include <qaction.h>
#include <qpixmap.h>
#include <q3vbox.h>
#include <qcursor.h>
#include <qstatusbar.h>
#include <q3filedialog.h>

#include "InitInterfaceDialog.h"
#include "CANopenAccess.h"
#include "cop/can_defs.h"
#include "tcp/tcp_defs.h"
#include <sys/socket.h>
#include <linux/can.h>

//////////////////////////////////////////////////////////////////////////////
//	class CMainFrame

CMainFrame::CMainFrame( QWidget* parent, const char* name, Qt::WFlags wflags )
	//: QMainWindow( 0, "main window", WDestructiveClose )
	: Q3MainWindow( parent, name, wflags )
{
	// attribute initialization
	m_CANopen = NULL;;

	// status bar and program icon
	(void)statusBar();
	if ( !name )
		setName( "main window" );
	setIcon( qPixmapFromMimeSource( PROGRAM_ICON ) );

	// user actions
	// - file open
	QAction* fileOpenAction = new QAction( this, "file open" );
	Q_CHECK_PTR( fileOpenAction );
	fileOpenAction->setText( "Open" );
	fileOpenAction->setMenuText( "&Open..." );
	fileOpenAction->setStatusTip( FILE_OPEN_STATUSTIP );
	fileOpenAction->setWhatsThis( FILE_OPEN_WHATsTHIS );
	fileOpenAction->setAccel( Qt::CTRL+Qt::Key_O );
	fileOpenAction->setIconSet( QIcon( qPixmapFromMimeSource( FILEOPEN_ICON ) ) );
	// - file print
#ifndef QT_NO_PRINTER
	m_filePrintAction = new QAction( this, "file print" );
	m_filePrintAction->setText( "Print" );
	m_filePrintAction->setMenuText( "&Print..." );
	m_filePrintAction->setStatusTip( FILE_PRINT_STATUSTIP );
	m_filePrintAction->setWhatsThis( FILE_PRINT_WHATsTHIS );
	m_filePrintAction->setAccel( Qt::CTRL+Qt::Key_P );
	m_filePrintAction->setIconSet( QIcon( qPixmapFromMimeSource( FILEPRINT_ICON ) ) );
#endif
	// - file close
	m_fileCloseAction = new QAction( this, "file close" );
	m_fileCloseAction->setText( "Close" );
	m_fileCloseAction->setMenuText( "&Close" );
	m_fileCloseAction->setStatusTip( FILE_CLOSE_STATUSTIP );
	m_fileCloseAction->setWhatsThis( FILE_CLOSE_WHATsTHIS );
	m_fileCloseAction->setAccel( Qt::CTRL+Qt::Key_W );
	m_fileCloseAction->setIconSet( QIcon( qPixmapFromMimeSource( FILECLOSE_ICON ) ) );
	// - file close all
	m_fileCloseAllAction = new QAction( this, "file close all" );
	m_fileCloseAllAction->setText( "Close All" );
	m_fileCloseAllAction->setMenuText( "C&lose All" );
	m_fileCloseAllAction->setStatusTip( FILE_CLOSE_ALL_STATUSTIP );
	m_fileCloseAllAction->setWhatsThis( FILE_CLOSE_ALL_WHATsTHIS );
	// - file exit
	QAction* fileExitAction = new QAction( this, "file exit" );
	fileExitAction->setText( "Exit" );
	fileExitAction->setMenuText( "E&xit" );
	fileExitAction->setStatusTip( FILE_EXIT_STATUSTIP );
	fileExitAction->setWhatsThis( FILE_EXIT_WHATsTHIS );
	fileExitAction->setAccel( Qt::CTRL+Qt::Key_Q );
	fileExitAction->setIconSet( QIcon( qPixmapFromMimeSource( EXIT_ICON ) ) );
	// - device info
	m_deviceInfoAction = new QAction( this, "device info" );
	m_deviceInfoAction->setText( "Device Info" );
	m_deviceInfoAction->setMenuText( "Device &Info" );
	m_deviceInfoAction->setStatusTip( DEVICE_INFO_STATUSTIP );
	m_deviceInfoAction->setWhatsThis( DEVICE_INFOT_WHATsTHIS );
	m_deviceInfoAction->setIconSet( QIcon( qPixmapFromMimeSource( DEVICEINFO_ICON ) ) );
	// - interface initialize
	m_interfaceInitializeAction = new QAction( this, "interface initialize" );
	m_interfaceInitializeAction->setText( "Initialize" );
	m_interfaceInitializeAction->setMenuText( "&Initialize..." );
	m_interfaceInitializeAction->setStatusTip( INTERFACE_INITIALIZE_STATUSTIP );
	m_interfaceInitializeAction->setWhatsThis( INTERFACE_INITIALIZE_WHATsTHIS );
	// - interface properties
	m_interfacePropertiesAction = new QAction( this, "interface properties" );
	m_interfacePropertiesAction->setText( "Properties" );
	m_interfacePropertiesAction->setMenuText( "&Properties..." );
	m_interfacePropertiesAction->setStatusTip( INTERFACE_PROPERTIES_STATUSTIP );
	m_interfacePropertiesAction->setWhatsThis( INTERFACE_PROPERTIES_WHATsTHIS );
	// - windows cascade
	m_windowsCascadeAction = new QAction( this, "file cascade" );
	m_windowsCascadeAction->setText( "Cascade" );
	m_windowsCascadeAction->setMenuText( "&Cascade" );
	m_windowsCascadeAction->setStatusTip( WINDOWS_CASCADE_STATUSTIP );
	m_windowsCascadeAction->setWhatsThis( WINDOWS_CASCADE_WHATsTHIS );
	// - windows tile
	m_windowsTileAction = new QAction( this, "file tile" );
	m_windowsTileAction->setText( "Tile" );
	m_windowsTileAction->setMenuText( "&Tile" );
	m_windowsTileAction->setStatusTip( WINDOWS_TILE_STATUSTIP );
	m_windowsTileAction->setWhatsThis( WINDOWS_TILE_WHATsTHIS );
	// - help topic
	QAction* helpTopicAction = new QAction( this, "help topic" );
	helpTopicAction->setText( "COPbrowser Handbook" );
	helpTopicAction->setMenuText( "COPbrowser &Handbook" );
	helpTopicAction->setStatusTip( HELP_CONTENTS_STATUSTIP );
	helpTopicAction->setWhatsThis( HELP_CONTENTS_WHATsTHIS );
	helpTopicAction->setAccel( Qt::Key_F1 );
	helpTopicAction->setIconSet( QIcon( qPixmapFromMimeSource( CONTENTS_ICON ) ) );
	// - help context
	QAction* helpWhatsThisAction = new QAction( this, "help context" );
	helpWhatsThisAction->setText( "What's This" );
	helpWhatsThisAction->setMenuText( "What's &This" );
	helpWhatsThisAction->setStatusTip( HELP_CONTEXTHELP_STATUSTIP );
	helpWhatsThisAction->setWhatsThis( HELP_CONTEXTHELP_WHATsTHIS );
	helpWhatsThisAction->setAccel( Qt::SHIFT+Qt::Key_F1 );
	helpWhatsThisAction->setIconSet( QIcon( qPixmapFromMimeSource( CONTEXTHELP_ICON ) ) );
	// - help about
	QAction* helpAboutAction = new QAction( this, "help about" );
	helpAboutAction->setText( "About COPbrowser" );
	helpAboutAction->setMenuText( "&About COPbrowser" );
	helpAboutAction->setStatusTip( HELP_ABOUT_STATUSTIP );
	helpAboutAction->setWhatsThis( HELP_ABOUT_WHATsTHIS );
	helpAboutAction->setIconSet( QIcon( qPixmapFromMimeSource( PROGRAM_ICON ) ) );
	// -help about qt
	QAction* helpAboutQtAction = new QAction( this, "help about qt" );
	helpAboutQtAction->setText( "About Qt" );
	helpAboutQtAction->setMenuText( "About &Qt" );
	helpAboutQtAction->setStatusTip( HELP_ABOUT_QT_STATUSTIP );
	helpAboutQtAction->setWhatsThis( HELP_ABOUT_QT_WHATsTHIS );

	// create the menu bar
	// - &File menu:
	m_fileMenu = new Q3PopupMenu( this );
	m_fileListMenu = new Q3PopupMenu( this );
	menuBar()->insertItem( "&File", m_fileMenu );
	fileOpenAction->addTo( m_fileMenu );
	m_fileMenu->insertItem( "&Recently used", m_fileListMenu );
	m_fileMenu->insertSeparator();
#ifndef QT_NO_PRINTER
	m_filePrintAction->addTo( m_fileMenu );
	m_fileMenu->insertSeparator();
#endif
	m_fileCloseAction->addTo( m_fileMenu );
	m_fileCloseAllAction->addTo( m_fileMenu );
	m_fileMenu->insertSeparator();
	fileExitAction->addTo( m_fileMenu );
	// - &Device menu:
	m_deviceMenu = new Q3PopupMenu( this );
	menuBar()->insertItem( "&Device", m_deviceMenu );
	// <add further device commands here>
	m_deviceInfoAction->addTo( m_deviceMenu );
	m_deviceInfoAction->setEnabled( false );
	// - &Interface menu:
	m_interfaceMenu = new Q3PopupMenu( this );
	menuBar()->insertItem( "&Interface", m_interfaceMenu );
	m_interfaceInitializeAction->addTo( m_interfaceMenu );
	m_interfaceMenu->insertSeparator();
	m_interfacePropertiesAction->addTo( m_interfaceMenu );
	// - &Windows menu:
	m_windowsMenu = new Q3PopupMenu( this );
	m_windowsMenu->setCheckable( TRUE );
	menuBar()->insertItem( "&Windows", m_windowsMenu );
	// - &Help menu:
	menuBar()->insertSeparator();
	Q3PopupMenu * helpMenu = new Q3PopupMenu( this );
	menuBar()->insertItem( "&Help", helpMenu );
	helpTopicAction->addTo( helpMenu );
	helpWhatsThisAction->addTo( helpMenu );
	helpMenu->insertSeparator();
	helpAboutAction->addTo( helpMenu );
	helpAboutQtAction->addTo( helpMenu );

	// create the file operations tooolbar
	Q3ToolBar* toolbarFiles = new Q3ToolBar( this );
	addToolBar( toolbarFiles, "File Operations", Qt::DockTop, TRUE );
	fileOpenAction->addTo( toolbarFiles );
	toolbarFiles->addSeparator();
	m_deviceInfoAction->addTo( toolbarFiles );
	toolbarFiles->addSeparator();
	helpWhatsThisAction->addTo( toolbarFiles );
	helpAboutAction->addTo( toolbarFiles );

	// create a workspace for the MDI child window(s)
	Q3VBox* vbox = new Q3VBox( this );
	vbox->setFrameStyle( Q3Frame::StyledPanel | Q3Frame::Sunken );
	m_workspace = new QWorkspace( vbox );
	m_workspace->setScrollBarsEnabled( TRUE );
	setCentralWidget( vbox );

	// signal/slot connections
	connect( fileOpenAction, SIGNAL( activated() ), this, SLOT( fileOpen() ) );
	connect( m_fileCloseAction, SIGNAL( activated() ), this, SLOT( fileClose() ) );
	connect( m_fileCloseAllAction, SIGNAL( activated() ), this, SLOT( fileCloseAll() ) );
#ifndef QT_NO_PRINTER
	connect( m_filePrintAction, SIGNAL( activated() ), this, SLOT( filePrint() ) );
#endif
	connect( fileExitAction, SIGNAL( activated() ), qApp, SLOT( closeAllWindows()) );
	connect( m_deviceInfoAction, SIGNAL( activated() ), this, SLOT( deviceInfo() ) );
	connect( m_interfaceInitializeAction, SIGNAL( activated() ), this, SLOT( interfaceInitialize() ) );
	connect( m_interfacePropertiesAction, SIGNAL( activated() ), this, SLOT( interfaceProperties() ) );
	connect( m_windowsCascadeAction, SIGNAL( activated() ), m_workspace, SLOT( cascade() ) );
	connect( m_windowsTileAction, SIGNAL( activated() ), m_workspace, SLOT( tile() ) );
	connect( helpTopicAction, SIGNAL( activated() ), this, SLOT( helpTopic() ) );
	connect( helpWhatsThisAction, SIGNAL( activated() ), this, SLOT( whatsThis() ) );
	connect( helpAboutAction, SIGNAL( activated() ), this, SLOT( helpAbout() ) );
	connect( helpAboutQtAction, SIGNAL( activated() ), this, SLOT( helpAboutQt() ) );
	connect( m_fileMenu, SIGNAL(aboutToShow()), this, SLOT(fileMenuAboutToShow()) );
	connect( m_fileListMenu, SIGNAL(aboutToShow()), this, SLOT(fileListMenuAboutToShow()) );
	connect( m_deviceMenu, SIGNAL(aboutToShow()), this, SLOT(deviceMenuAboutToShow()) );
	connect( m_interfaceMenu, SIGNAL(aboutToShow()), this, SLOT(interfaceMenuAboutToShow()) );
	connect( m_windowsMenu, SIGNAL(aboutToShow()), this, SLOT(windowsMenuAboutToShow()) );

	// ready for input
	statusBar()->message( "Ready", 2000 );
}

CMainFrame::~CMainFrame()
{
}

void CMainFrame::initializeInterface()
{
	interfaceInitialize();
}

void CMainFrame::setRecentFile( const QString& fileName )
{
	QStringList fileList;
	fileList.prepend( fileName );

	QSettings settings;
	settings.insertSearchPath( QSettings::Windows,  QString( "/%1" ).arg( WINDOWS_REGISTRY ) );
	for( size_t i = 0; i < maxRecentFiles; i++ ) {
		QString fileEntry = settings.readEntry( QString( "/%1/%2/%3%4" )
				.arg( APPLICATION_KEY )
				.arg( RECENT_FILES_KEY )
				.arg( RECENT_FILEi_KEY )
				.arg( i+1 ) );
		if( !fileEntry.isNull() && !fileEntry.isEmpty() && (fileEntry != fileName) )
			fileList.append( fileEntry );
	}
	for( size_t i = 0; i < fileList.size() && i < maxRecentFiles; i++ ) {
		settings.writeEntry( QString( "/%1/%2/%3%4" )
				.arg( APPLICATION_KEY )
				.arg( RECENT_FILES_KEY )
				.arg( RECENT_FILEi_KEY )
				.arg( i+1 ), fileList[ i ] );
	}
}

void CMainFrame::deleteFileName( const QString& fileName )
{
	QStringList fileList;
	//fileList.remove( fileName );

	QSettings settings;
	settings.insertSearchPath( QSettings::Windows,  QString( "/%1" ).arg( WINDOWS_REGISTRY ) );
	for( size_t i = 0; i < maxRecentFiles; i++ ) {
		QString fileEntry = settings.readEntry( QString( "/%1/%2/%3%4" )
				.arg( APPLICATION_KEY )
				.arg( RECENT_FILES_KEY )
				.arg( RECENT_FILEi_KEY )
				.arg( i+1 ) );
		if( !fileEntry.isNull() && !fileEntry.isEmpty() && (fileEntry != fileName) )
			fileList.append( fileEntry );
	}
	for( size_t i = 0; i < fileList.size() && i < maxRecentFiles; i++ ) {
		settings.writeEntry( QString( "/%1/%2/%3%4" )
				.arg( APPLICATION_KEY )
				.arg( RECENT_FILES_KEY )
				.arg( RECENT_FILEi_KEY )
				.arg( i+1 ), fileList[ i ] );
	}
}

CMDIWindow* CMainFrame::newDocument()
{
	CMDIWindow* window = new CMDIWindow( m_workspace, 0, Qt::WDestructiveClose );
	connect( window, SIGNAL( message(const QString&, int) ), statusBar(), SLOT( message(const QString&, int )) );
	window->setCaption( DOCUMENT_UNNAMED );
	window->setIcon( qPixmapFromMimeSource( DOCUMENT_ICON ) );
	// show the very first window in maximized mode
	if ( m_workspace->windowList().isEmpty() )
		window->showMaximized();
	else
		window->show();
	return window;
}

void CMainFrame::closeEvent( QCloseEvent* event )
{
	QWidgetList windows = m_workspace->windowList();
	if ( windows.count() ) {
		for ( int i = 0; i < int(windows.count()); ++i ) {
			QWidget *window = windows.at( i );
			if ( !window->close() ) {
				event->ignore();
				return;
			}
		}
	}
	if( m_CANopen )
	{
		m_CANopen->terminate();
		delete m_CANopen;
	}
	Q3MainWindow::closeEvent( event );
}

void CMainFrame::fileOpen()
{
	QString fileName = Q3FileDialog::getOpenFileName( QString::null, QString( DOCUMENT_FILTER ), this, "file open dialog" );
	if( !fileName.isEmpty() ) {
		CMDIWindow* window = newDocument();
		if( window->openDocument( fileName, m_CANopen ) ) {
			setRecentFile( fileName );
		}
		else {
			deleteFileName( fileName );
		}
		setCaption( APPLICATION_STRING );
	}
	else {
		statusBar()->message( "Operation aborted", 2000 );
	}
}

void CMainFrame::filePrint()
{
#ifndef QT_NO_PRINTER
	//@ToDo: print document
	//...
	QMessageBox::critical( this, PROGRAM_STRING" - Print", "Sorry, not realized yet!" );
#endif
}

void CMainFrame::fileClose()
{
	CMDIWindow* window = (CMDIWindow*)m_workspace->activeWindow();
	if( window )
		window->close();
}

void CMainFrame::fileCloseAll()
{
	QWidgetList windows = m_workspace->windowList();
	if ( windows.count() ) {
		for ( int i = 0; i < int(windows.count()); ++i ) {
			QWidget *window = windows.at( i );
			window->close();
		}
	}
}

void CMainFrame::deviceInfo()
{
	CMDIWindow* window = (CMDIWindow*)m_workspace->activeWindow();
	if( window )
		window->showDeviceInfo();
}

void CMainFrame::interfaceInitialize()
{
	struct _can_param can_param = {"nul", PF_CAN, SOCK_RAW, CAN_RAW};
	struct _top_param top_param = { 0x7F000001, 60000, {1, 1, 255} };
	short library = CCANopenAccess::libUndefined;
	long  ip1 = 127, ip2 = 0, ip3 = 0, ip4 = 1;
	long  port = 0;
	bool success = true;

	// Initialize Interface dialog
	CInitInterfaceDialog* dialog = new CInitInterfaceDialog( this );
	if( dialog->exec() != QDialog::Accepted )
	{
		return;
	}
	// get entered information
	const char* interface = dialog->getInterface();
	int baudrate = dialog->getBaudrate();
	if( interface )
	{
		if(sscanf(interface, "%li.%li.%li.%li:%li", &ip1, &ip2, &ip3, &ip4, &port) == 5) {
			library = CCANopenAccess::libCANopenOverTcpIp;
			top_param.addr = (unsigned long)(ip1 * (256*256*256) + ip2 * (256*256) + ip3 * (256) + ip4);
			top_param.port = (unsigned short)port;
		}
		else if(sscanf(interface, "localhost:%li", &port) == 1) {
			library = CCANopenAccess::libCANopenOverTcpIp;
			top_param.port = (unsigned short)port;
		}
		else {
			library = CCANopenAccess::libSocketCAN;
			can_param.ifname = (char*)interface;
		}
	}
	// initialize CAN interface
	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
	if( !m_CANopen )
	{
		m_CANopen = new CCANopenAccess;
	}
	else
	{
		m_CANopen->terminate();
	}
	Q_ASSERT( m_CANopen );

	if( library == CCANopenAccess::libCANopenOverTcpIp )
	{
		success = m_CANopen->initialize( library, CAN_REMOTE, &top_param, (unsigned char)baudrate );
	}
	else
	{
		success = m_CANopen->initialize( library, CAN_NETDEV, &can_param, (unsigned char)baudrate );
	}
	QApplication::restoreOverrideCursor();

	// ice hockey?
	if( success )
	{
		statusBar()->message( "CAN interface successfully initialized", 2000 );

		dialog->saveSettings();
	}
	else
	{
		QString err = QString( "Error %1: Interface '%2' could not be initialized!\n\n\t%3" )
								.arg( m_CANopen->getLastError() )
								.arg( interface )
								.arg( m_CANopen->getErrorText() );
		QMessageBox::critical( this, PROGRAM_STRING" - Initialize Interface", err);

		delete m_CANopen;
		m_CANopen = NULL;
	}
}

void CMainFrame::interfaceProperties()
{
	char  device[101], firmware[101], copyright[101];

	if( m_CANopen )
	{
		if( m_CANopen->getHardwareVersion( device, 100 ) &&
			m_CANopen->getFirmwareVersion( firmware, 100 ) &&
			m_CANopen->getCopyright( copyright, 100 ) )
		{
			QString msg = QString( "Hardware: %1\n" \
								   "Firmware: %2\n" \
								   "Software: %3\n" \
								   "Copyright %4\n" )
								   .arg( device )
								   .arg( firmware )
								   .arg( m_CANopen->getAPIVersion() )
								   .arg( copyright );
			QMessageBox::information( this, PROGRAM_STRING" - Interface Properties", msg );
		}
		else
		{
			QMessageBox::critical( this, PROGRAM_STRING" - Interface Properties",
										"Error ?: Properties could not be retrieved!");
		}
	}
	else
	{
		QMessageBox::warning( this, PROGRAM_STRING" - Interface Properties",
								   "Interface not initialized.\n\nProbably no board present." );
	}
}

void CMainFrame::helpTopic()
{
	//@ToDo: online help
	//...
	QMessageBox::critical( this, PROGRAM_STRING" - Help Topic", "Sorry, not realized yet!" );
}

void CMainFrame::helpAbout()
{
	int rev; if(sscanf(REVISION_NUMBER_STRING, "\044Rev: %i \044", &rev) != 1) rev = 0;

	QMessageBox::about( this, APPLICATION_STRING, QString( "%1 - %2 for Linux\n\n"
														   "Version %3 of "__DATE__" (Build %4)\n\n"
														   "   %5\n\n"
														   "Copyright %6\n\n"
														   "Internet URL: %7\n\n" )
															.arg( PROGRAM_STRING )
															.arg( APPLICATION_STRING )
															.arg( VERSION_NUMBER_STRING )
															.arg( rev )
															.arg( LICENSE_STRING )
															.arg( COPYRIGHT_STRING )
															.arg( INTERNET_STRING ) );
	//@ToDo: improve the about dialog!
}

void CMainFrame::helpAboutQt()
{
	QMessageBox::aboutQt( this, APPLICATION_STRING );
}

void CMainFrame::fileMenuAboutToShow()
{
	m_fileCloseAction->setEnabled( !m_workspace->windowList().isEmpty() );
	m_fileCloseAllAction->setEnabled( !m_workspace->windowList().isEmpty() );
#ifndef QT_NO_PRINTER
	m_filePrintAction->setEnabled( !m_workspace->windowList().isEmpty() );
#endif
}

void CMainFrame::fileListMenuAboutToShow()
{
	m_fileListMenu->clear();

	QStringList fileList;
	int count = 0;

	QSettings settings;
	settings.insertSearchPath( QSettings::Windows,  QString( "/%1" ).arg( WINDOWS_REGISTRY ) );
	for( size_t i = 0; i < maxRecentFiles; i++ ) {
		QString fileEntry = settings.readEntry( QString( "/%1/%2/%3%4" )
				.arg( APPLICATION_KEY )
				.arg( RECENT_FILES_KEY )
				.arg( RECENT_FILEi_KEY )
				.arg( i+1 ) );
		if( !fileEntry.isNull() && !fileEntry.isEmpty() ) {
			m_fileListMenu->insertItem( fileEntry, this, SLOT( fileListMenuActivated( int ) ) );
			count++;
		}
	}
	if( count == 0 ) {
		int id = m_fileListMenu->insertItem( "<recently used files>" );
		m_fileListMenu->setItemEnabled( id, FALSE );
	}
}

void CMainFrame::deviceMenuAboutToShow()
{
	m_deviceInfoAction->setEnabled( !m_workspace->windowList().isEmpty() );
}

void CMainFrame::interfaceMenuAboutToShow()
{
}

void CMainFrame::windowsMenuAboutToShow()
{
	m_windowsMenu->clear();

	m_windowsCascadeAction->addTo( m_windowsMenu );
	m_windowsTileAction->addTo( m_windowsMenu );

	m_windowsCascadeAction->setEnabled( !m_workspace->windowList().isEmpty() );
	m_windowsTileAction->setEnabled( !m_workspace->windowList().isEmpty() );

	if( !m_workspace->windowList().isEmpty() ) {
		m_windowsMenu->insertSeparator();
		QWidgetList windows = m_workspace->windowList();
		for( int i = 0; i < int(windows.count()); ++i ) {
			int id = m_windowsMenu->insertItem(windows.at(i)->caption(),
											   this, SLOT( windowsMenuActivated( int ) ) );
			m_windowsMenu->setItemParameter( id, i );
			m_windowsMenu->setItemChecked( id, m_workspace->activeWindow() == windows.at(i) );
		}
	}
}

void CMainFrame::fileListMenuActivated( int id )
{
	QString fileName = m_fileListMenu->text( id );
	if( !fileName.isEmpty() ) {
		CMDIWindow* window = newDocument();
		if( window->openDocument( fileName, m_CANopen ) ) {
			setRecentFile( fileName );
		}
		else {
			deleteFileName( fileName );
		}
		setCaption( APPLICATION_STRING );
	}
	else {
		statusBar()->message( "Operation aborted", 2000 );
	}
}

void CMainFrame::windowsMenuActivated( int id )
{
	QWidget* window = m_workspace->windowList().at( id );
	if ( window )
		window->showNormal();
	window->setFocus();
}

//////////////////////////////////////////////////////////////////////////////
//	revision control

const char* CMainFrame::version()
{
	return (const char*)_id;
}

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
