////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/MainFrame.h $
//
//	project   :  COPbrowser - CANopen Object Browser
//	
//	purpose   :  MainFrame - Main Window of the Application
//	
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CMainFrame
//	
//	includes  :  MDIWindow.h
//	
//	author    :  Uwe Vogt, UV Software
//	
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	MainFrame - Main Window of the Application

#ifndef _MainFrame_h_
#define _MainFrame_h_

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include <q3mainwindow.h>
//Added by qt3to4:
#include <Q3PopupMenu>
#include <QCloseEvent>
#include "MDIWindow.h"

//////////////////////////////////////////////////////////////////////////////
//	forwarder

class CCANopenAccess;
class QWorkspace;
class Q3PopupMenu;
class QAction;

//////////////////////////////////////////////////////////////////////////////
//	class CMainFrame

class CMainFrame : public Q3MainWindow
{
	Q_OBJECT
	
	//	attributes:
	private:
		QWorkspace* m_workspace;
		Q3PopupMenu* m_fileMenu;
		Q3PopupMenu* m_fileListMenu;
		Q3PopupMenu* m_deviceMenu;
		Q3PopupMenu* m_interfaceMenu;
		Q3PopupMenu* m_windowsMenu;
		QAction* m_fileCloseAction;
		QAction* m_fileCloseAllAction;
#ifndef QT_NO_PRINTER
		QAction* m_filePrintAction;
#endif
		QAction* m_deviceInfoAction;
		QAction* m_interfaceInitializeAction;
		QAction* m_interfacePropertiesAction;
		QAction* m_windowsCascadeAction;
		QAction* m_windowsTileAction;
		CCANopenAccess* m_CANopen;
		
		enum  { maxRecentFiles = 10 };
	
	//	constructor/destructor:
	public:
		CMainFrame( QWidget* parent = 0, const char* name = 0, Qt::WFlags wflags = Qt::WType_TopLevel );
		~CMainFrame();
	
	//	member functions:
	protected:
		void closeEvent( QCloseEvent* );
	
	private slots:
		CMDIWindow* newDocument();
		
		void fileOpen();
		void filePrint();
		void fileClose();
		void fileCloseAll();
		//void fileExit();
		void deviceInfo();
		void interfaceInitialize();
		void interfaceProperties();
		void helpTopic();
		void helpAbout();
		void helpAboutQt();
		
		void fileMenuAboutToShow();
		void fileListMenuAboutToShow();
		void deviceMenuAboutToShow();
		void interfaceMenuAboutToShow();
		void windowsMenuAboutToShow();
		void fileListMenuActivated( int id );
		void windowsMenuActivated( int id );

	public:
		void initializeInterface();

	private:
		void setRecentFile( const QString& fileName );
		void deleteFileName( const QString& fileName );
	
	//	revision control:
	public:
		static const char* version();
};

#endif // _MainFrame_h_

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
