////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/dic/CANopenDeviceInfo.h $
//
//	project   :  CAN - Controller Area Network
//
//	purpose   :  CANopen Device Info
//
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CCANopenDeviceInfo
//
//	includes  :  (none)
//
//	author    :  Uwe Vogt, UV Software
//
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	CANopen Device Info

#ifndef _CANopenDeviceInfo_h_
#define _CANopenDeviceInfo_h_

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include <qstring.h>

//////////////////////////////////////////////////////////////////////////////
//	defines

#define MAX_BAUDRATE	9				// Baudrate-table: number of entries

//////////////////////////////////////////////////////////////////////////////
//	class CCANopenDeviceInfo

class CCANopenDeviceInfo
{
	//	enumerations:
	public:
		enum TBaudrateIndex {			// Indexes for the baudrate-table:
			baudrate1000 = 0,			//   Baudrate 1000 kBit/s
			baudrate800 = 1,			//   Baudrate 800 kBit/s
			baudrate500 = 2,			//   Baudrate 500 kBit/s
			baudrate250 = 3,			//   Baudrate 250 kBit/s
			baudrate125 = 4,			//   Baudrate 125 kBit/s
			baudrate100 = 5,			//   Baudrate 100 kBit/s
			baudrate50 = 6,				//   Baudrate 50 kBit/s
			baudrate20 = 7,				//   Baudrate 20 kBit/s
			baudrate10 = 8,				//   Baudrate 10 kBit/s
		};
		enum TDeviceInfoState {			// State of the device-info:
			statePublic = 0,			//   Public device-info
			stateProtected,				//   Protected device-info
			statePrivate,				//   Private device-info
			stateInvalid = -1,			//   Invalid device-info
		};
	//	attributes:
	private:
		QString m_vendorName;			// Name of the vendor
		QString m_productName;			// Name of the product
		unsigned long m_vendorNumber;	// LSS vendor-id.
		unsigned long  m_productNumber;	// LSS product code
		unsigned long  m_revisionNumber;// LSS product revision
		QString m_orderCode;			// Order code for the product
		bool    m_baudrate[MAX_BAUDRATE];//Table of baudrates
		struct tSimpleBootUp {			// Simple boot-up functionality
			bool    m_master;			//   Simple boot-up master
			bool    m_slave;			//   Simple boot-up slave
		}	m_simpleBootUp;
		struct tPDO {					// PDO settings:
			unsigned short m_rxPDO;		//   Number of receive PDOs
			unsigned short m_txPDO;		//   Number of transmit PDOs
			unsigned char  m_granularity;//  Granularity allowed for mapping
		}	m_PDO;
		bool    m_dynamicChannels;		// Facility of dynamic variable generation
		bool    m_groupMessaging;		// Facility of multiplexed PDOs
		bool    m_layerSettings;		// Layer Settings Specification
		QString m_lmtManufacturerName;	// LMT manufacturer name (DS-301, V3.0)
		QString m_lmtProductName;		// LMT product name (DS-301, V3.0)
		QString m_EDSVersion;			// Version of EDS-file specification ("4.0")
		QString	m_description;			// A brief description of the device (uv-extension)

	//	constructor/destructor:
	public:
		CCANopenDeviceInfo();
		~CCANopenDeviceInfo();

	//	member functions:
	public:
		QString getVendorName()				{ return m_vendorName; }
		QString getProductName()			{ return m_productName; }
		unsigned long getVendorNumber()		{ return m_vendorNumber; }
		unsigned long getProductNumber()	{ return m_productNumber; }
		unsigned long getRevisionNumber()	{ return m_revisionNumber; }
		QString getOrderCode()				{ return m_orderCode; }
		bool    getBaudrate( TBaudrateIndex index )	{
			if((0 <= index) && (index < MAX_BAUDRATE))
				return m_baudrate[index];
			else
				return false;
		};
		bool    getSimpleBootUpMaster()		{ return m_simpleBootUp.m_master; }
		bool    getSimpleBootUpSlave()		{ return m_simpleBootUp.m_slave; }
		unsigned short getNumberOfRxPDO()	{ return m_PDO.m_rxPDO; }
		unsigned short getNumberOfTxPDO()	{ return m_PDO.m_txPDO; }
		unsigned char  getPDOGranularity()	{ return m_PDO.m_granularity; }
		bool    getDynamicChannels()		{ return m_dynamicChannels; }
		bool    getGroupMessaging()			{ return m_groupMessaging; }
		bool    getLayerSettings()			{ return m_layerSettings; }
		QString getLmtManufacturerName()	{ return m_lmtManufacturerName; }
		QString getLmtProductName()			{ return m_lmtProductName; }
		QString getEDSVersion()				{ return m_EDSVersion; }
		QString	getDescription()			{ return m_description; };

	protected:
		friend class CCANopenReader;		// access by class CCANopenReader allowed

		void setVendorName( const char* name )			{ m_vendorName = name; }
		void setProductName( const char* name )			{ m_productName = name; }
		void setVendorNumber( unsigned long number )	{ m_vendorNumber = number; }
		void setProductNumber( unsigned long number )	{ m_productNumber = number; }
		void setRevisionNumber( unsigned long number )	{ m_revisionNumber = number; }
		void setOrderCode( const char* orderCode )		{ m_orderCode = orderCode; }
		void setBaudrate(int index, bool supported )	{
			if((0 <= index) && (index < MAX_BAUDRATE))
				this->m_baudrate[index] = supported;
		};
		void setSimpleBootUpMaster( bool supported )	{ m_simpleBootUp.m_master = supported; }
		void setSimpleBootUpSlave( bool supported )		{ m_simpleBootUp.m_slave = supported; }
		void setNumberOfRxPDO( unsigned short rxPDO )	{ m_PDO.m_rxPDO = rxPDO; }
		void setNumberOfTxPDO( unsigned short txPDO )	{ m_PDO.m_txPDO = txPDO; }
		void setPDOGranularity( unsigned char granularity )	{ m_PDO.m_granularity = granularity; }
		void setDynamicChannels( bool supported )		{ m_dynamicChannels = supported; }
		void setGroupMessaging( bool supported )		{ m_groupMessaging = supported; }
		void setLayerSettings( bool supported )			{ m_layerSettings = supported; }
		void setLmtManufacturerName( const char* name )	{ m_lmtManufacturerName = name; }
		void setLmtProductName( const char* name )		{ m_lmtProductName = name; }
		void setEDSVersion( const char* version)		{ m_EDSVersion = version; }
		void setDescription( const char* text )			{ m_description += text; };

	//	static member
	public:
		static QString getBaudrateByIndex( TBaudrateIndex index );
		static QString getBooleanByValue( bool boolean );
		static QString getStateByIndex( TDeviceInfoState index );
		static int  getBaudrateByName( const char* name );
		static bool getBooleanByName( const char* name );
		static int  getStateByName( const char* name );

	//	revision control:
	public:
		static const char* version();
};

#endif // _CANopenDeviceInfo_h_

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
