////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/dic/CANopenObjectEntry.cpp $
//
//	project   :  CAN - Controller Area Network
//
//	purpose   :  CANopen Object Entry
//
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CCANopenObjectEntry
//
//	includes  :  CANopenObjectEntry.h
//
//	author    :  Uwe Vogt, UV Software
//
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	CANopen Object Entry

static const char _id[] = "$Id: CANopenObjectEntry.cpp 56 2009-11-05 18:48:27Z saturn $";

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "CANopenObjectEntry.h"

//////////////////////////////////////////////////////////////////////////////
//	class CCANopenObjectEntry

CCANopenObjectEntry::CCANopenObjectEntry( unsigned short index, unsigned char subIndex )
{
	m_index = index;					// Profile index number
	m_subIndex = subIndex;				// Profile sub-index number
	m_name = "";						// Name of the parameter
	m_dataType = typeVoid;				// Data type classification
	m_category = categoryOptional;		// 'Optional' or 'Mandatory' entry
	m_access = accessCONST;				// Access attribute of the parameter
	m_PDOMapping = mappingNo;			// Mapping option of the parameter
	m_lowLimit = "";					// Value range: Low limit value (optional)
	m_highLimit = "";					// Value range: High limit value (optional)
	m_defaultValue = "";				// Default value of the parameter (optional)
	m_objectFlags = 0x00000000UL;		// Object flags for specific behaviour (optional)
}

CCANopenObjectEntry::CCANopenObjectEntry()
{
	m_index = 0x0000;					// Profile index number
	m_subIndex = 0xFF;					// Profile sub-index number
	m_name = "";						// Name of the parameter
	m_dataType = typeVoid;				// Data type classification
	m_category = categoryOptional;		// 'Optional' or 'Mandatory' entry
	m_access = accessCONST;				// Access attribute of the parameter
	m_PDOMapping = mappingNo;			// Mapping option of the parameter
	m_lowLimit = "";					// Value range: Low limit value (optional)
	m_highLimit = "";					// Value range: High limit value (optional)
	m_defaultValue = "";				// Default value of the parameter (optional)
	m_objectFlags = 0x00000000UL;		// Object flags for specific behaviour (optional)
}

CCANopenObjectEntry::~CCANopenObjectEntry()
{
}

QString CCANopenObjectEntry::getValueRange()
{
	if( !m_lowLimit.isEmpty() && !m_highLimit.isEmpty() )
		return QString( "%1,..,%2" ).arg( m_lowLimit ).arg( m_highLimit );
	else if( !m_lowLimit.isEmpty() )
		return QString( "%1,..." ).arg( m_lowLimit );
	else if( !m_highLimit.isEmpty() )
		return QString( "...,%2" ).arg( m_lowLimit ).arg( m_highLimit );
	else
		return getDataTypeByIndex(this->m_dataType);
}

QString CCANopenObjectEntry::getStateByIndex( TEntryState index )
{
	QString strObjectEntryType;

	switch(index)						// State of an object:
	{
		case CCANopenObjectEntry::statePublic:	//   Public  object
			strObjectEntryType = "Public";
			break;
		case CCANopenObjectEntry::stateProtected://  Protected object
			strObjectEntryType = "Protected";
			break;
		case CCANopenObjectEntry::statePrivate:	//   Private object
			strObjectEntryType = "Private";
			break;
		case CCANopenObjectEntry::stateInvalid:	//   Invalid object
			strObjectEntryType = "Invalid";
			break;
		default:							//   Sollte eigentlich nie vorkommen!
			strObjectEntryType = "Invalid";
			break;
	}
	return strObjectEntryType;
}

QString CCANopenObjectEntry::getDataTypeByIndex( TEntryType index )
{
	QString strObjectEntryType;

	switch(index)						// Type attribute of an object entry:
	{
		case CCANopenObjectEntry::typeBoolean:	// Boolean
			strObjectEntryType = "Boolean";
			break;
		case CCANopenObjectEntry::typeInteger8:	// Integer8
			strObjectEntryType = "Integer8";
			break;
		case CCANopenObjectEntry::typeInteger16:// Integer16
			strObjectEntryType = "Integer16";
			break;
		case CCANopenObjectEntry::typeInteger32:// Integer32
			strObjectEntryType = "Integer32";
			break;
		case CCANopenObjectEntry::typeUnsigned8:// Unsigned8
			strObjectEntryType = "Unsigned8";
			break;
		case CCANopenObjectEntry::typeUnsigned16://Unsigned16
			strObjectEntryType = "Unsigned16";
			break;
		case CCANopenObjectEntry::typeUnsigned32://Unsigned32
			strObjectEntryType = "Unsigned32";
			break;
		case CCANopenObjectEntry::typeReal32:	// Real32
			strObjectEntryType = "Real32";
			break;
		case CCANopenObjectEntry::typeVisibleString://Visible String
			strObjectEntryType = "Visible String";
			break;
			case CCANopenObjectEntry::typeOctetString://  Octet String
				strObjectEntryType = "Octet String";
				break;
#ifdef _UNICODE_STRING
		case CCANopenObjectEntry::typeUnicodeString://Unicode String
			strObjectEntryType = "Unicode String";
			break;
#else
		case CCANopenObjectEntry::typeDate:	// Date
			strObjectEntryType = "Date";
			break;
#endif
		case CCANopenObjectEntry::typeTimeOfDay:// Time Of Day
			strObjectEntryType = "Time Of Day";
			break;
		case CCANopenObjectEntry::typeTimeDifference://Time Difference
			strObjectEntryType = "Time Difference";
			break;
		case CCANopenObjectEntry::typeDomain:	// Domain
			strObjectEntryType = "Domain";
			break;
		case CCANopenObjectEntry::typeInteger24:// Integer24
			strObjectEntryType = "Integer24";
			break;
		case CCANopenObjectEntry::typeReal64:	// Real64
			strObjectEntryType = "Real64";
			break;
		case CCANopenObjectEntry::typeInteger40:// Integer40
			strObjectEntryType = "Integer40";
			break;
		case CCANopenObjectEntry::typeInteger48:// Integer48
			strObjectEntryType = "Integer48";
			break;
		case CCANopenObjectEntry::typeInteger56:// Integer56
			strObjectEntryType = "Integer56";
			break;
		case CCANopenObjectEntry::typeInteger64:// Integer64
			strObjectEntryType = "Integer64";
			break;
		case CCANopenObjectEntry::typeUnsigned24://Unsigned24
			strObjectEntryType = "Unsigned24";
			break;
		case CCANopenObjectEntry::typeUnsigned40://Unsigned40
			strObjectEntryType = "Unsigned40";
			break;
		case CCANopenObjectEntry::typeUnsigned48://Unsigned48
			strObjectEntryType = "Unsigned48";
			break;
		case CCANopenObjectEntry::typeUnsigned56://Unsigned56
			strObjectEntryType = "Unsigned56";
			break;
		case CCANopenObjectEntry::typeUnsigned64://Unsigned64
			strObjectEntryType = "Unsigned64";
			break;
		default:								// Benutzer-definierter Basis-Datentyp
			strObjectEntryType = "User-defined Type";
			break;
	}
	return strObjectEntryType;
}

QString CCANopenObjectEntry::getCategoryByIndex( TEntryCategory index )
{
	QString strObjectEntryCategory;

	switch(index)						// Category of an object entry:
	{
		case CCANopenObjectEntry::categoryMandatory:	// Mandatory object
			strObjectEntryCategory = "Mandatory";
			break;
		case CCANopenObjectEntry::categoryConditional:	// Conditional object
			strObjectEntryCategory = "Conditional";
			break;
		case CCANopenObjectEntry::categoryOptional:		// Optional object
			strObjectEntryCategory = "Optional";
			break;
		default:										// Sollte eigentlich nie vorkommen!
			strObjectEntryCategory = "Optional";
			break;
	}
	return strObjectEntryCategory;
}

QString CCANopenObjectEntry::getAccessByIndex( TEntryAccess index )
{
	QString strObjectEntryAccess;

	switch(index)						// Access attribute of an object entry:
	{
		case CCANopenObjectEntry::accessCONST:	// CONST  = Read-only access, value is constant
			strObjectEntryAccess = "CONST";
			break;
		case CCANopenObjectEntry::accessRO:		// RO     = Read-only access
			strObjectEntryAccess = "RO";
			break;
		case CCANopenObjectEntry::accessWO:		// WO     = Write-only access
			strObjectEntryAccess = "WO";
			break;
		case CCANopenObjectEntry::accessRW:		// RW     = Read and write access
			strObjectEntryAccess = "RW";
			break;
		case CCANopenObjectEntry::accessRWR:	// RWR    = Read/write on process input
			strObjectEntryAccess = "RWR";
			break;
		case CCANopenObjectEntry::accessRWW:	// RWW    = Read/write on process output
			strObjectEntryAccess = "RWW";
			break;
		default:								// Sollte eigentlich nie vorkommen!
			strObjectEntryAccess = "";
			break;
	}
	return strObjectEntryAccess;
}

QString CCANopenObjectEntry::getMappingByIndex( TEntryMapping index )
{
	QString strObjectEntryMapping;

	switch(index)						// Mapping option of an object entry:
	{
		case CCANopenObjectEntry::mappingOptional:	// Optional  = Object may be mapped into a PDO
			strObjectEntryMapping = "Optional";
			break;
		case CCANopenObjectEntry::mappingDefault:	// Default   = Object is part of the default mapping
			strObjectEntryMapping = "Default";
			break;
		case CCANopenObjectEntry::mappingNo:		// No        = Object must not be mapped into a PDO
		default:
			strObjectEntryMapping = "No";
			break;
	}
	return strObjectEntryMapping;
}

QString CCANopenObjectEntry::getStorableByIndex( TEntryStorable index )
{
	QString strObjectEntryStorable;

	switch(index)						// Storing option of an object entry:
	{
		case CCANopenObjectEntry::storableYes:	// Yes   = Object is stored on request
			strObjectEntryStorable = "Yes";
			break;
		case CCANopenObjectEntry::storableAuto:	// Auto  = Object is stored autonomously
			strObjectEntryStorable = "Auto";
			break;
		case CCANopenObjectEntry::storableNo:	// No    = Object can not be stored
		default:
			strObjectEntryStorable = "No";
			break;
	}
	return strObjectEntryStorable;
}

int CCANopenObjectEntry::getStateByName( const char* name )
{
	if(!strcasecmp(name, "Public"))		//   Public object
		return (int)CCANopenObjectEntry::statePublic;
	else if(!strcasecmp(name, "Protected"))// Protected object
		return (int)CCANopenObjectEntry::stateProtected;
	else if(!strcasecmp(name, "Private"))//   Private object
		return (int)CCANopenObjectEntry::statePrivate;
	else								//   Invalid object
		return (int)CCANopenObjectEntry::stateInvalid;
}

int CCANopenObjectEntry::getDataTypeByName( const char* name )
{
	if(!strcasecmp(name, "Boolean"))
		return (int)CCANopenObjectEntry::typeBoolean;
	else if(!strcasecmp(name, "Integer8"))
		return (int)CCANopenObjectEntry::typeInteger8;
	else if(!strcasecmp(name, "Integer16"))
		return (int)CCANopenObjectEntry::typeInteger16;
	else if(!strcasecmp(name, "Integer32"))
		return (int)CCANopenObjectEntry::typeInteger32;
	else if(!strcasecmp(name, "Unsigned8"))
		return (int)CCANopenObjectEntry::typeUnsigned8;
	else if(!strcasecmp(name, "Unsigned16"))
		return (int)CCANopenObjectEntry::typeUnsigned16;
	else if(!strcasecmp(name, "Unsigned32"))
		return (int)CCANopenObjectEntry::typeUnsigned32;
	else if(!strcasecmp(name, "Real32"))
		return (int)CCANopenObjectEntry::typeReal32;
	else if(!strcasecmp(name, "Visible String"))
		return (int)CCANopenObjectEntry::typeVisibleString;
	else if(!strcasecmp(name, "Octet String"))
		return (int)CCANopenObjectEntry::typeOctetString;
#ifdef _UNICODE_STRING
	else if(!strcasecmp(name, "Unicode String"))
		return (int)CCANopenObjectEntry::typeUnicodeString;
#else
	else if(!strcasecmp(name, "Date"))
		return (int)CCANopenObjectEntry::typeDate;
#endif
	else if(!strcasecmp(name, "Time Of Day"))
		return (int)CCANopenObjectEntry::typeTimeOfDay;
	else if(!strcasecmp(name, "Time Difference"))
		return (int)CCANopenObjectEntry::typeTimeDifference;
	else if(!strcasecmp(name, "Domain"))
		return (int)CCANopenObjectEntry::typeDomain;
	else if(!strcasecmp(name, "Integer24"))
		return (int)CCANopenObjectEntry::typeInteger24;
	else if(!strcasecmp(name, "Real64"))
		return (int)CCANopenObjectEntry::typeReal64;
	else if(!strcasecmp(name, "Integer40"))
		return (int)CCANopenObjectEntry::typeInteger40;
	else if(!strcasecmp(name, "Integer48"))
		return (int)CCANopenObjectEntry::typeInteger48;
	else if(!strcasecmp(name, "Integer56"))
		return (int)CCANopenObjectEntry::typeInteger56;
	else if(!strcasecmp(name, "Integer64"))
		return (int)CCANopenObjectEntry::typeInteger64;
	else if(!strcasecmp(name, "Unsigned24"))
		return (int)CCANopenObjectEntry::typeUnsigned24;
	else if(!strcasecmp(name, "Unsigned40"))
		return (int)CCANopenObjectEntry::typeUnsigned40;
	else if(!strcasecmp(name, "Unsigned48"))
		return (int)CCANopenObjectEntry::typeUnsigned48;
	else if(!strcasecmp(name, "Unsigned56"))
		return (int)CCANopenObjectEntry::typeUnsigned56;
	else if(!strcasecmp(name, "Unsigned64"))
		return (int)CCANopenObjectEntry::typeUnsigned64;
	else
		return (int)CCANopenObjectEntry::typeVoid;
}

int CCANopenObjectEntry::getCategoryByName( const char* name )
{
	if(!strncasecmp(name, "Optional", strlen("Optional")))
		return (int)CCANopenObjectEntry::categoryOptional;
	else if(!strncasecmp(name, "Mandatory", strlen("Mandatory")))
		return (int)CCANopenObjectEntry::categoryMandatory;
	else if(!strncasecmp(name, "Conditional", strlen("Conditional")))
		return (int)CCANopenObjectEntry::categoryConditional;
	else								// Sollte eigentlich nie vorkommen!
		return (int)CCANopenObjectEntry::categoryOptional;
}

int CCANopenObjectEntry::getAccessByName( const char* name )
{
	if(!strcasecmp(name, "CONST"))		// CONST  = Read-only access, value is constant
		return (int)CCANopenObjectEntry::accessCONST;
	else if(!strcasecmp(name, "RO"))   	// RO     = Read-only access
		return (int)CCANopenObjectEntry::accessRO;
	else if(!strcasecmp(name, "WO"))   	// WO     = Write-only access
		return (int)CCANopenObjectEntry::accessWO;
	else if(!strcasecmp(name, "RW"))   	// RW     = Read and write access
		return (int)CCANopenObjectEntry::accessRW;
	else if(!strcasecmp(name, "RWR"))  	// RWR    = Read/write on process input
		return (int)CCANopenObjectEntry::accessRWR;
	else if(!strcasecmp(name, "RWW"))  	// RWW    = Read/write on process output
		return (int)CCANopenObjectEntry::accessRWW;
	else								// Sollte eigentlich nie vorkommen!
		return (int)CCANopenObjectEntry::accessCONST;
}

int CCANopenObjectEntry::getMappingByName( const char* name )
{
	if(!strncasecmp(name, "Optional", strlen("Optional")))
		return (int)CCANopenObjectEntry::mappingOptional;
	else if(!strncasecmp(name, "Possible", strlen("Possible")))
		return (int)CCANopenObjectEntry::mappingOptional;
	else if(!strncasecmp(name, "Default", strlen("Default")))
		return (int)CCANopenObjectEntry::mappingDefault;
	else if(!strncasecmp(name, "Yes", strlen("Yes")))
		return (int)CCANopenObjectEntry::mappingDefault;
	else if(!strncasecmp(name, "No", strlen("No")))
		return (int)CCANopenObjectEntry::mappingNo;
	else if(!strcmp(name, "0"))
		return (int)CCANopenObjectEntry::mappingNo;
	else if(!strcmp(name, "1"))
		return (int)CCANopenObjectEntry::mappingDefault;
	else								// Sollte eigentlich nie vorkommen!
		return (int)CCANopenObjectEntry::mappingNo;
}

int CCANopenObjectEntry::getStorableByName( const char* name )
{
	if(!strncasecmp(name, "Yes", strlen("Yes")))
		return (int)CCANopenObjectEntry::storableYes;
	else if(!strncasecmp(name, "Auto", strlen("Auto")))
		return (int)CCANopenObjectEntry::storableAuto;
	else if(!strncasecmp(name, "No", strlen("No")))
		return (int)CCANopenObjectEntry::storableNo;
	else if(!strcmp(name, "0"))
		return (int)CCANopenObjectEntry::storableNo;
	else if(!strcmp(name, "1"))
		return (int)CCANopenObjectEntry::storableYes;
	else								// Sollte eigentlich nie vorkommen!
		return (int)CCANopenObjectEntry::storableNo;
}

//////////////////////////////////////////////////////////////////////////////
//	revision control

const char* CCANopenObjectEntry::version()
{
	return (const char*)_id;
}

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
