////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/dic/CANopenObjectEntry.h $
//
//	project   :  CAN - Controller Area Network
//
//	purpose   :  CANopen Object Entry
//
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CCANopenObjectEntry
//
//	includes  :  (none)
//
//	author    :  Uwe Vogt, UV Software
//
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	CANopen Object Entry

#ifndef _CANopenObjectEntry_h_
#define _CANopenObjectEntry_h_

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include <qstring.h>

//////////////////////////////////////////////////////////////////////////////
//	class CCANopenObjectEntry

class CCANopenObjectEntry
{
	//	enumerations:
	public:
		enum TEntryType {				// Type attribute of an object entry:
			typeBoolean = 0x1,			//   Boolean
			typeInteger8 = 0x2,			//   Integer8
			typeInteger16 = 0x3,		//   Integer16
			typeInteger32 = 0x4,		//   Integer32
			typeUnsigned8 = 0x5,		//   Unsigned8
			typeUnsigned16 = 0x6,		//   Unsigned16
			typeUnsigned32 = 0x7,		//   Unsigned32
			typeReal32 = 0x8,			//   Real32
			typeVisibleString = 0x9,	//   Visible String
			typeOctetString = 0xA,		//   Octet String
#ifdef _UNICODE_STRING					//   #DS-301, V4.0
			typeUnicodeString = 0xB,	//    Unicode String
#else									//   #DS-301, V3.0
			typeDate = 0xB,				//    Date
#endif									//   #for compatibility!
			typeTimeOfDay = 0xC,		//   Time Of Day
			typeTimeDifference = 0xD,	//   Time Difference
			typeDomain = 0xF,			//   Domain
			typeInteger24 = 0x10,		//   Integer24
			typeReal64 = 0x11,			//   Real64
			typeInteger40 = 0x12,		//   Integer40
			typeInteger48 = 0x13,		//   Integer48
			typeInteger56 = 0x14,		//   Integer56
			typeInteger64 = 0x15,		//   Integer64
			typeUnsigned24 = 0x16,		//   Unsigned24
			typeUnsigned40 = 0x18,		//   Unsigned40
			typeUnsigned48 = 0x19,		//   Unsigned48
			typeUnsigned56 = 0x1A,		//   Unsigned56
			typeUnsigned64 = 0x1B,		//   Unsigned64
			typePDOParameter = 0x20,	//   PDO CommPar
			typePDOMapping = 0x21,		//   PDO Mapping
			typeSDOParameter = 0x22,	//   SDO Parameter
			typeIdentity = 0x23,		//   Identity
			typeVoid = 0,				//   'universal' data type
			//typeUserDefined = -1,		//   User-defined data type
		};
		enum TEntryCategory {			// Category of an object entry:
			categoryOptional = 0,		//   Optional object
			categoryMandatory = 1,		//   Mandatory object
			categoryConditional,		//   Conditional object
		};
		enum TEntryAccess {				// Access attribute of an object entry:
			accessCONST = 0,			//   CONST  = Read-only access, value is constant
			accessRO,					//   RO     = Read-only access
			accessWO,					//   WO     = Write-only access
			accessRW,					//   RW     = Read and write access
			accessRWR,					//   RWR    = Read/write on process input
			accessRWW,					//   RWW    = Read/write on process output
		};
		enum TEntryMapping {			// Mapping option of an object entry:
			mappingOptional = 2,		//   Optional  = Object may be mapped into a PDO
			mappingDefault = 1,			//   Default   = Object is part of the default mapping
			mappingNo = 0,				//   No        = Object must not be mapped into a PDO
		};
		enum TEntryStorable {			// Storing option of an object entry:
			storableYes = 1,			//   Yes   = Object is stored on request
			storableAuto,				//   Auto  = Object is stored autonomously
			storableNo = 0,				//   No    = Object can not be stored
		};
		enum TEntryState {				// State of an object entry:
			statePublic = 0,			//   Public object entry
			stateProtected,				//   Protected object entry
			statePrivate,				//   Private object entry
			stateInvalid = -1,			//   Invalid object entry
		};
	//	attributes:
	private:
		unsigned short	m_index;		// Profile index number
		unsigned char	m_subIndex;		// Profile sub-index number
		QString			m_name;			// Name of the parameter
		TEntryType		m_dataType;		// Data type classification
		TEntryCategory	m_category;		// 'Optional' or 'Mandatory' entry
		TEntryAccess	m_access;		// Access attribute of the parameter
		TEntryMapping	m_PDOMapping;	// Mapping option of the parameter
		QString			m_lowLimit;	// Value range: Low limit value (optional)
		QString			m_highLimit;	// Value range: High limit value (optional)
		QString			m_defaultValue;	// Default value of the parameter (optional)
		unsigned long	m_objectFlags;	// Object flags for specific behaviour (optional)

	//	constructor/destructor:
	public:
		CCANopenObjectEntry( unsigned short index, unsigned char subIndex );
		~CCANopenObjectEntry();
	protected:
		CCANopenObjectEntry();

	//	member functions:
	public:
		unsigned short	getIndex()			{ return m_index; };
		unsigned char	getSubindex()		{ return m_subIndex; };
		QString			getName()			{ return m_name; };
		TEntryType		getDataType()		{ return m_dataType; };
		TEntryCategory	getCategory()		{ return m_category; };
		TEntryAccess	getAccess()			{ return m_access; };
		TEntryMapping	getPDOMapping()		{ return m_PDOMapping; };
		QString			getLowLimit()		{ return m_lowLimit; };
		QString			getHighLimit()		{ return m_highLimit; };
		QString			getValueRange();
		QString			getDefaultValue()	{ return m_defaultValue; };
		unsigned long   getObjectFlags()	{ return m_objectFlags; };

	protected:
		friend class CCANopenReader;		// access by class CCANopenReader allowed

		void setName( const char* name )	{ m_name = name; }
		void setDataType( int dataType )	{ m_dataType = (TEntryType)dataType; };
		void setCategory( int category )	{ m_category = (TEntryCategory)category; };
		void setAccess(int accessType )		{ m_access = (TEntryAccess)accessType; };
		void setPDOMapping(int PDOMapping)		{ m_PDOMapping = (TEntryMapping)PDOMapping; };
		void setLowLimit( const char* lowLimit )	{ m_lowLimit = lowLimit; };
		void setHighLimit( const char* highLimit )		{ m_highLimit = highLimit; };
		void setDefaultValue( const char* defaultValue )	{ m_defaultValue = defaultValue; };
		void setObjectFlags( unsigned long objectFlags )	{ m_objectFlags = objectFlags; };

	//	static member
	public:
		static QString getStateByIndex( TEntryState index );
		static QString getDataTypeByIndex( TEntryType index );
		static QString getCategoryByIndex( TEntryCategory index );
		static QString getAccessByIndex( TEntryAccess index );
		static QString getMappingByIndex( TEntryMapping index );
		static QString getStorableByIndex( TEntryStorable index );
		static int getStateByName( const char* name );
		static int getDataTypeByName( const char* name );
		static int getCategoryByName( const char* name );
		static int getAccessByName( const char* name );
		static int getMappingByName( const char* name );
		static int getStorableByName( const char* name );

	//	revision control:
	public:
		static const char* version();
};

#endif // _CANopenObjectEntry_h_

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
