////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/dic/CANopenReader.cpp $
//
//	project   :  CAN - Controller Area Network
//
//	purpose   :  CANopen Files (Input)
//
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CCANopenReader
//
//	includes  :  CANopenReader.h (CANopen.h ), IniReader.h
//
//	author    :  Uwe Vogt, UV Software
//
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	CANopen Files (Input)

static const char _id[] = "$Id: CANopenReader.cpp 56 2009-11-05 18:48:27Z saturn $";

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "CANopenReader.h"
#include "IniReader.h"

//////////////////////////////////////////////////////////////////////////////
//	prototypes for helper functions

static bool IsObject(const char* lpszAscii, unsigned short* puiIndex);
static bool IsObjectEntry(const char* lpszAscii, unsigned short* puiIndex, unsigned char* pucSubIndex);
//static bool Ascii2Index(const char* lpszAscii, unsigned short* pValue);
//static bool Ascii2Subindex(const char* lpszAscii, unsigned char* pValue);
//static bool Ascii2Integer8(const char* lpszAscii, char* pValue);
//static bool Ascii2Integer16(const char* lpszAscii, short* pValue);
//static bool Ascii2Integer32(const char* lpszAscii, long* pValue);
static bool Ascii2Unsigned8(const char* lpszAscii, unsigned char* pValue);
static bool Ascii2Unsigned16(const char* lpszAscii, unsigned short* pValue);
static bool Ascii2Unsigned32(const char* lpszAscii, unsigned long* pValue);
//static char ReadTextValue(CStdioFile* fpFile, CString& strValue);


//////////////////////////////////////////////////////////////////////////////
//	class CCANopenReader

CCANopenReader::CCANopenReader( CCANopen* objectDictionary, CCANopen::TFileType fileType )
{
	Q_CHECK_PTR( objectDictionary );	// NULL pointer assignment?
	m_dictionary = objectDictionary;	// CANopen object dictionary
	m_fileType = fileType;				// Type of the CANopen input file
}

CCANopenReader::CCANopenReader()
{
}

CCANopenReader::~CCANopenReader()
{
}

int CCANopenReader::load( const char* fileName )
{
	if( !m_dictionary )					// NULL pointer assignment?
		return false;
	m_dictionary->m_fileName = fileName;	// Name of the CANopen input file
	m_dictionary->m_fileType = m_fileType;	// Type of the CANopen input file

	switch( m_fileType )
	{
		case CCANopen::typeEDS:			//   EDS (CiA - Electronic Data Sheet)
			return loadEDS( fileName );
		case CCANopen::typeDCF:			//   DCF (CiA - Device Configuration File)
			return loadEDS( fileName );
		case CCANopen::typeXML:			//   XML (UVS - eXtensible Markup Language)
			return loadXML( fileName );
		case CCANopen::typeCOP:			//   COP (UVS - CANopen Objektedatei, ASCII)
			return loadCOP( fileName );
		default:						//   unknown file type!
			return false;
	}
}

// / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / / /
// CANopen EDS-file (CiA - Electronic Data Sheet)

#define EdsFileInfo				0
#define EdsDeviceInfo			1
#define EdsMandatoryObjects		2
#define EdsOptionalObjects		3
#define EdsManufacturerObjects	4
#define EdsObject				5
#define EdsObjectEntry			6
#define EdsUnknown				-1


int CCANopenReader::loadEDS( const char* fileName )
{
	int nSection;						// [<Section>]
	int nItem;							// <key>=<value>

	// open the EDS-file (.ini-style) for reading
	CIniReader* ptrIniFile = new CIniReader( fileName );
	if( ptrIniFile == NULL )
		return false;

	// mother´s little helper
	int     nPos     = -1;
	unsigned char  ucNumber  = 0;
	unsigned short uiNumber  = 0;
	unsigned long  ulNumber  = 0;
	QString strEdsVersion = "4.0";
	QString strDescription = "";
	// - CANopen Objekt
	CCANopenObject* ptrCANopenObject = NULL;
	// - CANopen Objekteintrag
	CCANopenObjectEntry* ptrCANopenObjectEntry = NULL;
	// - CANopen Gerätebeschreibung
	CCANopenDeviceInfo* ptrCANopenDeviceInfo = NULL;

	// load the object dictionary from the EDS-file
	while((nItem = ptrIniFile->moveToNextItem()) != CIniReader::None)
	{
		switch(nItem)
		{
			case CIniReader::Section:		// INI-Reader hat einen neuen Abschnitt gelesen
				if(ptrIniFile->getSection().compare/*NoCase*/("FileInfo") == 0)
				{
					nSection = EdsFileInfo;
				}
				else if(ptrIniFile->getSection().compare/*NoCase*/("DeviceInfo") == 0)
				{
					nSection = EdsDeviceInfo;

					// CANopen Gerätebeschreibung anlegen (falls noch nicht geschehen)
					if(ptrCANopenDeviceInfo == NULL)
					{
						ptrCANopenDeviceInfo = new CCANopenDeviceInfo();
						// und dem Objektverzeichnis hinzufügen
						m_dictionary->m_deviceInfo = ptrCANopenDeviceInfo;
						// Versionsnummer für´s EDS-File
						ptrCANopenDeviceInfo->setEDSVersion(strEdsVersion);
						ptrCANopenDeviceInfo->setDescription(strDescription);
					}
				}
				else if(ptrIniFile->getSection().compare/*NoCase*/("MandatoryObjects") == 0)
				{
					nSection = EdsMandatoryObjects;
				}
				else if(ptrIniFile->getSection().compare/*NoCase*/("OptionalObjects") == 0)
				{
					nSection = EdsOptionalObjects;
				}
				else if(ptrIniFile->getSection().compare/*NoCase*/("ManufacturerObjects") == 0)
				{
					nSection = EdsManufacturerObjects;
				}
				else
				{
					if(IsObject(ptrIniFile->getSection(), &uiNumber))
					{
						nSection = EdsObject;

					// CANopen Objekt im Objektverzeichnis suchen
						if(ptrCANopenObject != NULL)
						{
							if(ptrCANopenObject->getIndex() != uiNumber)
								if((nPos = m_dictionary->find(uiNumber)) != -1)
									ptrCANopenObject = m_dictionary->m_objectArray[nPos];
						}
						else
						{
							if((nPos = m_dictionary->find(uiNumber)) != -1)
								ptrCANopenObject = m_dictionary->m_objectArray[nPos];
						}
					// CANopen Objekt mit Index anlegen (falls nicht gefunden)
						if(ptrCANopenObject == NULL)
						{
							ptrCANopenObject = new CCANopenObject(uiNumber);
						// und am Ende des Objektverzeichnisses hinzufügen
							m_dictionary->m_objectArray.resize(m_dictionary->m_objectArray.count() + 1);
							m_dictionary->m_objectArray.insert(m_dictionary->m_objectArray.count(), ptrCANopenObject);
						}
					// Kein aktueller Objekteintrag!
						ptrCANopenObjectEntry = NULL;
					}
					else if(IsObjectEntry(ptrIniFile->getSection(), &uiNumber, &ucNumber))
					{
						nSection = EdsObjectEntry;

					// CANopen Objekt im Objektverzeichnis suchen
						if(ptrCANopenObject != NULL)
						{
							if(ptrCANopenObject->getIndex() != uiNumber)
								if((nPos = m_dictionary->find(uiNumber)) != -1)
									ptrCANopenObject = m_dictionary->m_objectArray[nPos];
						}
						else
						{
							if((nPos = m_dictionary->find(uiNumber)) != -1)
								ptrCANopenObject = m_dictionary->m_objectArray[nPos];
						}
					// CANopen Objekt mit Index anlegen (falls nicht gefunden)
						if(ptrCANopenObject == NULL)
						{
							ptrCANopenObject = new CCANopenObject(uiNumber);
						// und am Ende des Objektverzeichnisses hinzufügen
							m_dictionary->m_objectArray.resize(m_dictionary->m_objectArray.count() + 1);
							m_dictionary->m_objectArray.insert(m_dictionary->m_objectArray.count(), ptrCANopenObject);
						}
					// Objekteintrag mit Index und Subindex anlegen
						ptrCANopenObjectEntry = new CCANopenObjectEntry(uiNumber, ucNumber);
					// und am Ende des Objektverzeichnisses hinzufügen
						ptrCANopenObject->m_objectEntryArray.resize(ptrCANopenObject->m_objectEntryArray.count() + 1);
						ptrCANopenObject->m_objectEntryArray.insert(ptrCANopenObject->m_objectEntryArray.count(), ptrCANopenObjectEntry);
					// Implizite Eintragsattributte setzen
						if(ptrCANopenObjectEntry->getSubindex() == 0)
							ptrCANopenObjectEntry->setCategory(CCANopenObjectEntry::categoryMandatory);
						else
							ptrCANopenObjectEntry->setCategory(ptrCANopenObject->getCategory());
					}
					else
					{
						nSection = EdsUnknown;
					}
				}
				break;
			case CIniReader::Item:			// INI-Reader hat einen neuen Eintrag gelesen
				break;
			default:						// INI-Reader hat einen Fehler erkannt
				delete ptrIniFile;
				return false;
		}
		switch(nSection)
		{
			case EdsFileInfo:
				if(ptrIniFile->getKeyName().compare/*NoCase*/("EDSVersion") == 0)
				{
					strEdsVersion = ptrIniFile->getKeyValue();
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("FileVersion") == 0)
				{
					m_dictionary->m_info.m_revision = ptrIniFile->getKeyValue();
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("Description") == 0)
				{
					strDescription = ptrIniFile->getKeyValue();
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("CreationTime") == 0)
				{
					m_dictionary->m_info.m_time = ptrIniFile->getKeyValue();
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("CreationDate") == 0)
				{
					m_dictionary->m_info.m_date = ptrIniFile->getKeyValue();
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("CreatedBy") == 0)
				{
					m_dictionary->m_info.m_author = ptrIniFile->getKeyValue();
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("ModificationTime") == 0)
				{
					if(!ptrIniFile->getKeyValue().isEmpty())
						m_dictionary->m_info.m_time = ptrIniFile->getKeyValue();
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("ModificationDate") == 0)
				{
					if(!ptrIniFile->getKeyValue().isEmpty())
						m_dictionary->m_info.m_date = ptrIniFile->getKeyValue();
				}
				//else if(ptrIniFile->getKeyName().compare/*NoCase*/("ModifiedBy") == 0)
				//{
				//	m_dictionary->m_info.m_Author = ptrIniFile->getKeyValue();
				//}
				break;
			case EdsDeviceInfo:
				if(ptrIniFile->getKeyName().compare/*NoCase*/("VendorName") == 0)
				{
				// Name des Herstellers
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setVendorName(ptrIniFile->getKeyValue());
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("ProductName") == 0)
				{
				// Name des Produkts
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setProductName(ptrIniFile->getKeyValue());
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("VendorNumber") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned32(ptrIniFile->getKeyValue(), &ulNumber))
						ulNumber = 0x0UL;
				// LSS Vendor-Id. des Geräts
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setVendorNumber(ulNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("ProductNumber") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned32(ptrIniFile->getKeyValue(), &ulNumber))
						ulNumber = 0x0UL;
				// LSS Vendor-Id. des Geräts
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setProductNumber(ulNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("RevisionNumber") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned32(ptrIniFile->getKeyValue(), &ulNumber))
						ulNumber = 0x0UL;
				// LSS Revision-No. des Geräts
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setRevisionNumber(ulNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("OrderCode") == 0)
				{
				// Bestell-/Ident.-Nummer
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setOrderCode(ptrIniFile->getKeyValue());
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("BaudRate_10") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Baudrate 10 kBit/s?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setBaudrate(8, uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("BaudRate_20") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Baudrate 20 kBit/s?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setBaudrate(7, uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("BaudRate_50") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Baudrate 50 kBit/s?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setBaudrate(6, uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("BaudRate_100") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Baudrate 100 kBit/s?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setBaudrate(5, uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("BaudRate_125") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Baudrate 125 kBit/s?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setBaudrate(4, uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("BaudRate_250") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Baudrate 250 kBit/s?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setBaudrate(3, uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("BaudRate_500") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Baudrate 500 kBit/s?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setBaudrate(2, uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("BaudRate_800") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Baudrate 800 kBit/s?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setBaudrate(1, uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("BaudRate_1000") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Baudrate 1000 kBit/s?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setBaudrate(0, uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("SimpleBootUpMaster") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Simple boot-up master?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setSimpleBootUpMaster(uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("SimpleBootUpSlave") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Simple boot-up slave?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setSimpleBootUpSlave(uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("Granularity") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned8(ptrIniFile->getKeyValue(), &ucNumber))
						ucNumber = 0;
				// PDO Mapping Granularität
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setPDOGranularity(ucNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("DynamicChannelsSupported") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Dynamic channels?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setDynamicChannels(uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("GroupMessaging") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Group messaging?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setGroupMessaging(uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("NrOfRXPDO") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Anzahl Receive PDOs
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setNumberOfRxPDO(uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("NrOfTXPDO") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Anzahl Transmit PDOs
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setNumberOfTxPDO(uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("LSS_Supported") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Layer settings service?
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setLayerSettings(uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("LMT_ManufacturerName") == 0)
				{
				// LMT Manufacturer name
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setLmtManufacturerName(ptrIniFile->getKeyValue());
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("LMT_ProductName") == 0)
				{
				// LMT Product name
					if(ptrCANopenDeviceInfo != NULL)
						ptrCANopenDeviceInfo->setLmtProductName(ptrIniFile->getKeyValue());
				}
				break;
			case EdsMandatoryObjects:
				if(ptrIniFile->getKeyName().compare/*NoCase*/("SupportedObjects") == 0)
				{
				}
				else
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0x0000;
					if(uiNumber != 0x0000)
					{
					// Neues CANopen Objekt mit Index anlegen
						ptrCANopenObject = new CCANopenObject(uiNumber);
					// und am Ende des Objektverzeichnisses hinzufügen
						m_dictionary->m_objectArray.resize(m_dictionary->m_objectArray.count() + 1);
						m_dictionary->m_objectArray.insert(m_dictionary->m_objectArray.count(), ptrCANopenObject);
					// Inplizite Objektattributte setzen
						if(uiNumber < 0x1000)
						{
							ptrCANopenObject->setProfileNumber("CiA DS-301");
							ptrCANopenObject->setProfileChapter("Data type entry");
						}
						else if(0x1000 <= uiNumber && uiNumber < 0x2000)
						{
							ptrCANopenObject->setProfileNumber("CiA DS-301");
							ptrCANopenObject->setProfileChapter("Communication profile");
						}
						else if(0x2000 <= uiNumber && uiNumber < 0x6000)
						{
							ptrCANopenObject->setProfileNumber("Manufacturer specific");
						}
						else if(0x6000 <= uiNumber && uiNumber < 0xA000)
						{
							ptrCANopenObject->setProfileNumber("Device profile");
						}
						else
						{
							ptrCANopenObject->setProfileNumber("(reserved)");
						}
						ptrCANopenObject->setCategory(CCANopenObject::categoryMandatory);
					}
					else// Sonst: kein gültiger Objekt-Zeiger
					{
						ptrCANopenObject = NULL;
					}
				}
				break;
			case EdsOptionalObjects:
				if(ptrIniFile->getKeyName().compare/*NoCase*/("SupportedObjects") == 0)
				{
				}
				else
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0x0000;
					if(uiNumber != 0x0000)
					{
					// Neues CANopen Objekt mit Index anlegen
						ptrCANopenObject = new CCANopenObject(uiNumber);
					// und am Ende des Objektverzeichnisses hinzufügen
						m_dictionary->m_objectArray.resize(m_dictionary->m_objectArray.count() + 1);
						m_dictionary->m_objectArray.insert(m_dictionary->m_objectArray.count(), ptrCANopenObject);
					// Inplizite Objektattributte setzen
						if(uiNumber < 0x1000)
						{
							ptrCANopenObject->setProfileNumber("CiA DS-301");
							ptrCANopenObject->setProfileChapter("Data type entry");
						}
						else if(0x1000 <= uiNumber && uiNumber < 0x2000)
						{
							ptrCANopenObject->setProfileNumber("CiA DS-301");
							ptrCANopenObject->setProfileChapter("Communication profile");
						}
						else if(0x2000 <= uiNumber && uiNumber < 0x6000)
						{
							ptrCANopenObject->setProfileNumber("Manufacturer specific");
						}
						else if(0x6000 <= uiNumber && uiNumber < 0xA000)
						{
							ptrCANopenObject->setProfileNumber("Device profile");
						}
						else
						{
							ptrCANopenObject->setProfileNumber("(reserved)");
						}
						ptrCANopenObject->setCategory(CCANopenObject::categoryOptional);
					}
					else// Sonst: kein gültiger Objekt-Zeiger
					{
						ptrCANopenObject = NULL;
					}
				}
				break;
			case EdsManufacturerObjects:
				if(ptrIniFile->getKeyName().compare/*NoCase*/("SupportedObjects") == 0)
				{
				}
				else
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0x0000;
					if(uiNumber != 0x0000)
					{
					// Neues CANopen Objekt mit Index anlegen
						ptrCANopenObject = new CCANopenObject(uiNumber);
					// und am Ende des Objektverzeichnisses hinzufügen
						m_dictionary->m_objectArray.resize(m_dictionary->m_objectArray.count() + 1);
						m_dictionary->m_objectArray.insert(m_dictionary->m_objectArray.count(), ptrCANopenObject);
					// Inplizite Objektattributte setzen
						if(uiNumber < 0x1000)
						{
							ptrCANopenObject->setProfileNumber("CiA DS-301");
							ptrCANopenObject->setProfileChapter("Data type entry");
						}
						else if(0x1000 <= uiNumber && uiNumber < 0x2000)
						{
							ptrCANopenObject->setProfileNumber("CiA DS-301");
							ptrCANopenObject->setProfileChapter("Communication profile");
						}
						else if(0x2000 <= uiNumber && uiNumber < 0x6000)
						{
							ptrCANopenObject->setProfileNumber("Manufacturer specific");
						}
						else if(0x6000 <= uiNumber && uiNumber < 0xA000)
						{
							ptrCANopenObject->setProfileNumber("Device profile");
						}
						else
						{
							ptrCANopenObject->setProfileNumber("(reserved)");
						}
						ptrCANopenObject->setCategory(CCANopenObject::categoryOptional);
					}
					else// Sonst: kein gültiger Objekt-Zeiger
					{
						ptrCANopenObject = NULL;
					}
				}
				break;
			case EdsObject:
				if(ptrIniFile->getKeyName().compare/*NoCase*/("ParameterName") == 0)
				{
				// Name des Objekts
					if(ptrCANopenObject != NULL)
						ptrCANopenObject->setName(ptrIniFile->getKeyValue());
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("ObjectType") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned8(ptrIniFile->getKeyValue(), &ucNumber))
						ucNumber = (unsigned char)CCANopenObject::classNULL;
				// Variablen-Klassifikation des Objekts
					if(ptrCANopenObject != NULL)
					{
						ptrCANopenObject->setClass(ucNumber);

						if(!ptrCANopenObject->isMultipleObject())
						{
						// Neuen Objekteintrag mit Index und Subindex anlegen
							ptrCANopenObjectEntry = new CCANopenObjectEntry(ptrCANopenObject->getIndex(), 0);
						// und Eintrag am Ende des aktuellen Objekts hinzufügen
							ptrCANopenObject->m_objectEntryArray.resize(ptrCANopenObject->m_objectEntryArray.count() + 1);
							ptrCANopenObject->m_objectEntryArray.insert(ptrCANopenObject->m_objectEntryArray.count(), ptrCANopenObjectEntry);
						// Implizite Eintragsattributte setzen
							ptrCANopenObjectEntry->setName(ptrCANopenObject->getName());
							ptrCANopenObjectEntry->setCategory(ptrCANopenObject->getCategory());
						}
					}
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("DataType") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = (unsigned short)CCANopenObject::typeVoid;
				// Datentyp des Objekts
					if(ptrCANopenObject != NULL)
						ptrCANopenObject->setDataType(uiNumber);
				// Datentyp des Objekteintrags
					if(ptrCANopenObjectEntry != NULL)
						ptrCANopenObjectEntry->setDataType(uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("AccessType") == 0)
				{
				// Zugriffsattribut des Objekteintrags
					if(ptrCANopenObjectEntry != NULL)
						ptrCANopenObjectEntry->setAccess(
								CCANopenObjectEntry::getAccessByName(
								ptrIniFile->getKeyValue()
																	)	);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("PDOMapping") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Mapping-Attribut des Objekteintrags
					if(ptrCANopenObjectEntry != NULL)
						if(uiNumber)
							ptrCANopenObjectEntry->setPDOMapping(CCANopenObjectEntry::mappingDefault);
					else
						ptrCANopenObjectEntry->setPDOMapping(CCANopenObjectEntry::mappingNo);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("LowLimit") == 0)
				{
				// Kleinster Wert (als Zeichenkette)
					if(ptrCANopenObjectEntry != NULL)
						ptrCANopenObjectEntry->setLowLimit(ptrIniFile->getKeyValue());
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("HighLimit") == 0)
				{
				// Größter Wert (als Zeichenkette)
					if(ptrCANopenObjectEntry != NULL)
						ptrCANopenObjectEntry->setHighLimit(ptrIniFile->getKeyValue());
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("DefaultValue") == 0)
				{
				// Default-Wert (als Zeichenkette)
					if(ptrCANopenObjectEntry != NULL)
						ptrCANopenObjectEntry->setDefaultValue(ptrIniFile->getKeyValue());
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("ObjFlags") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned32(ptrIniFile->getKeyValue(), &ulNumber))
						ulNumber = 0;
				// Flags des Objekteintrags
					if(ptrCANopenObjectEntry != NULL)
						ptrCANopenObjectEntry->setObjectFlags(ulNumber);
				}
				break;
			case EdsObjectEntry:
				if(ptrIniFile->getKeyName().compare/*NoCase*/("ParameterName") == 0)
				{
				// Name des Objekteintrags
					if(ptrCANopenObjectEntry != NULL)
						ptrCANopenObjectEntry->setName(ptrIniFile->getKeyValue());
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("ObjectType") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned8(ptrIniFile->getKeyValue(), &ucNumber))
						ucNumber = (unsigned char)CCANopenObject::classNULL;
				// Variablen-Klassifikation des Objekteintrags
				// [Nicht vorgesehen in CCANopenObjectEntry!]
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("DataType") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(),  &uiNumber))
						uiNumber = (unsigned short)CCANopenObject::typeVoid;
				// Datentyp des Objekteintrags
					if(ptrCANopenObjectEntry != NULL)
						ptrCANopenObjectEntry->setDataType(uiNumber);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("AccessType") == 0)
				{
				// Zugriffsattribut des Objekteintrags
					if(ptrCANopenObjectEntry != NULL)
						ptrCANopenObjectEntry->setAccess(
								CCANopenObjectEntry::getAccessByName(
								ptrIniFile->getKeyValue()
																	)	);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("PDOMapping") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (dez.)
					if(!Ascii2Unsigned16(ptrIniFile->getKeyValue(), &uiNumber))
						uiNumber = 0;
				// Mapping-Attribut des Objekteintrags
					if(ptrCANopenObjectEntry != NULL)
						if(uiNumber)
							ptrCANopenObjectEntry->setPDOMapping(CCANopenObjectEntry::mappingDefault);
					else
						ptrCANopenObjectEntry->setPDOMapping(CCANopenObjectEntry::mappingNo);
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("LowLimit") == 0)
				{
				// Kleinster Wert (als Zeichenkette)
					if(ptrCANopenObjectEntry != NULL)
						ptrCANopenObjectEntry->setLowLimit(ptrIniFile->getKeyValue());
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("HighLimit") == 0)
				{
				// Größter Wert (als Zeichenkette)
					if(ptrCANopenObjectEntry != NULL)
						ptrCANopenObjectEntry->setHighLimit(ptrIniFile->getKeyValue());
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("DefaultValue") == 0)
				{
				// Default-Wert (als Zeichenkette)
					if(ptrCANopenObjectEntry != NULL)
						ptrCANopenObjectEntry->setDefaultValue(ptrIniFile->getKeyValue());
				}
				else if(ptrIniFile->getKeyName().compare/*NoCase*/("ObjFlags") == 0)
				{
				// Zeichenkette in ganze Zahl konvertieren (hex. oder dez.)
					if(!Ascii2Unsigned32(ptrIniFile->getKeyValue(), &ulNumber))
						ulNumber = 0;
				// Flags des Objekteintrags
					if(ptrCANopenObjectEntry != NULL)
						ptrCANopenObjectEntry->setObjectFlags(ulNumber);
				}
				break;
		}
	}
	// close EDS-file and return with success
	delete ptrIniFile;
	return true;
}

int CCANopenReader::loadXML( const char* fileName )
{
	//@ToDo: load XML file
	//...
	if( fileName )
	{}
	return false;
}

int CCANopenReader::loadCOP( const char* fileName )
{
	//@ToDo: load COP file
	//...
	if( fileName )
	{}
	return false;
}

//////////////////////////////////////////////////////////////////////////////
//	helper functions

static bool IsObject(const char* lpszAscii, unsigned short* puiIndex)
{
	int d = 0;
	const char* ptr;
	unsigned short index = 0;

	Q_ASSERT(lpszAscii);
	Q_ASSERT(puiIndex);

	*puiIndex = index;
	ptr = lpszAscii;

	if(*ptr == '\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == '\0')
			return false;
		if(*ptr == 'x' || *ptr == 'X' || *ptr == '0') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			if(*ptr == '\0')
				return false;
			for(d = 0; *ptr != '\0'; ptr++, d++) {
				if(d == 4)
					return false;
				if('0' <= *ptr && *ptr <= '9')
					index = (index << 4) + (unsigned short)(*ptr - '0');
				else if('a' <= *ptr && *ptr <= 'f')
					index = (index << 4) + (unsigned short)(*ptr - 'a' + 10);
				else if('A' <= *ptr && *ptr <= 'F')
					index = (index << 4) + (unsigned short)(*ptr - 'A' + 10);
				else
					return false;
			}
			if(*ptr != '\0')
				return false;
			*puiIndex = index;
			// error 12: illegal character!
			return false;
		}
		return false;
	}
	else if(('1' <= *ptr && *ptr <= '9') ||
				('a' <= *ptr && *ptr <= 'f') ||
				('A' <= *ptr && *ptr <= 'F')) {
		for(d = 0; *ptr != '\0'; ptr++, d++) {
			if(d == 4)
				return false;
			if('0' <= *ptr && *ptr <= '9')
				index = (index << 4) + (unsigned short)(*ptr - '0');
			else if('a' <= *ptr && *ptr <= 'f')
				index = (index << 4) + (unsigned short)(*ptr - 'a' + 10);
			else if('A' <= *ptr && *ptr <= 'F')
				index = (index << 4) + (unsigned short)(*ptr - 'A' + 10);
			else
				return false;
		}
		if(*ptr != '\0')
			return false;
		*puiIndex = index;
		return true;
				}
				return false;
}

static bool IsObjectEntry(const char* lpszAscii, unsigned short* puiIndex, unsigned char* pucSubIndex)
{
	int d = 0;
	const char* ptr;
	unsigned short index = 0;
	unsigned char subindex = 0;

	Q_ASSERT(lpszAscii);
	Q_ASSERT(puiIndex);
	Q_ASSERT(pucSubIndex);

	ptr = lpszAscii;
	*puiIndex = index;
	*pucSubIndex = subindex;

	if(*ptr == '\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == '\0')
			return false;
		if(*ptr == 'x' || *ptr == 'X' || *ptr == '0') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			if(*ptr == '\0')
				return false;
			for(d = 0; *ptr != '\0'; ptr++, d++) {
				if(d == 4)
					break;;
				if('0' <= *ptr && *ptr <= '9')
					index = (index << 4) + (unsigned short)(*ptr - '0');
				else if('a' <= *ptr && *ptr <= 'f')
					index = (index << 4) + (unsigned short)(*ptr - 'a' + 10);
				else if('A' <= *ptr && *ptr <= 'F')
					index = (index << 4) + (unsigned short)(*ptr - 'A' + 10);
				else
					return false;
			}
			if(*ptr != 's' && *ptr != 'S')
				return false;
			else
				ptr++;
			if(*ptr != 'u' && *ptr != 'U')
				return false;
			else
				ptr++;
			if(*ptr != 'b' && *ptr != 'B')
				return false;
			else
				ptr++;
			if(*ptr == '0') {
				if(*++ptr == '\0') {
					*puiIndex = index;
					*pucSubIndex = subindex;
					// error 12: illegal character!
					return false;
				}
				if(*ptr == 'x' || *ptr == 'X' || *ptr == '0') {
					for(++ptr; *ptr == '0'; ptr++)
						;
					if(*ptr == '\0')
						return false;
					for(d = 0; *ptr != '\0'; ptr++, d++) {
						if(d == 2)
							return false;
						if('0' <= *ptr && *ptr <= '9')
							subindex = (subindex << 4) + (unsigned short)(*ptr - '0');
						else if('a' <= *ptr && *ptr <= 'f')
							subindex = (subindex << 4) + (unsigned short)(*ptr - 'a' + 10);
						else if('A' <= *ptr && *ptr <= 'F')
							subindex = (subindex << 4) + (unsigned short)(*ptr - 'A' + 10);
						else
							return false;
					}
					if(*ptr != '\0')
						return false;
					*puiIndex = index;
					*pucSubIndex = subindex;
					// error 12: illegal character!
					return false;
				}
				return false;
			}
			else if(('1' <= *ptr && *ptr <= '9') ||
							  ('a' <= *ptr && *ptr <= 'f') ||
							  ('A' <= *ptr && *ptr <= 'F')) {
				for(d = 0; *ptr != '\0'; ptr++, d++) {
					if(d == 2)
						return false;
					if('0' <= *ptr && *ptr <= '9')
						subindex = (subindex << 4) + (unsigned short)(*ptr - '0');
					else if('a' <= *ptr && *ptr <= 'f')
						subindex = (subindex << 4) + (unsigned short)(*ptr - 'a' + 10);
					else if('A' <= *ptr && *ptr <= 'F')
						subindex = (subindex << 4) + (unsigned short)(*ptr - 'A' + 10);
					else
						return false;
				}
				if(*ptr != '\0')
					return false;
				*puiIndex = index;
				*pucSubIndex = subindex;
				// error 12: illegal character!
				return false;
							  }
							  return false;
		}
		return false;
	}
	else if(('1' <= *ptr && *ptr <= '9') ||
				('a' <= *ptr && *ptr <= 'f') ||
				('A' <= *ptr && *ptr <= 'F')) {
		for(d = 0; *ptr != '\0'; ptr++, d++) {
			if(d == 4)
				break;;
			if('0' <= *ptr && *ptr <= '9')
				index = (index << 4) + (unsigned short)(*ptr - '0');
			else if('a' <= *ptr && *ptr <= 'f')
				index = (index << 4) + (unsigned short)(*ptr - 'a' + 10);
			else if('A' <= *ptr && *ptr <= 'F')
				index = (index << 4) + (unsigned short)(*ptr - 'A' + 10);
			else
				return false;
		}
		if(*ptr != 's' && *ptr != 'S')
			return false;
		else
			ptr++;
		if(*ptr != 'u' && *ptr != 'U')
			return false;
		else
			ptr++;
		if(*ptr != 'b' && *ptr != 'B')
			return false;
		else
			ptr++;
		if(*ptr == '0') {
			if(*++ptr == '\0') {
				*puiIndex = index;
				*pucSubIndex = subindex;
				return true;
			}
			if(*ptr == 'x' || *ptr == 'X' || *ptr == '0') {
				for(++ptr; *ptr == '0'; ptr++)
					;
				if(*ptr == '\0')
					return false;
				for(d = 0; *ptr != '\0'; ptr++, d++) {
					if(d == 2)
						return false;
					if('0' <= *ptr && *ptr <= '9')
						subindex = (subindex << 4) + (unsigned short)(*ptr - '0');
					else if('a' <= *ptr && *ptr <= 'f')
						subindex = (subindex << 4) + (unsigned short)(*ptr - 'a' + 10);
					else if('A' <= *ptr && *ptr <= 'F')
						subindex = (subindex << 4) + (unsigned short)(*ptr - 'A' + 10);
					else
						return false;
				}
				if(*ptr != '\0')
					return false;
				*puiIndex = index;
				*pucSubIndex = subindex;
				// error 12: illegal character!
				return false;
			}
			return false;
		}
		else if(('1' <= *ptr && *ptr <= '9') ||
					   ('a' <= *ptr && *ptr <= 'f') ||
					   ('A' <= *ptr && *ptr <= 'F')) {
			for(d = 0; *ptr != '\0'; ptr++, d++) {
				if(d == 2)
					return false;
				if('0' <= *ptr && *ptr <= '9')
					subindex = (subindex << 4) + (unsigned short)(*ptr - '0');
				else if('a' <= *ptr && *ptr <= 'f')
					subindex = (subindex << 4) + (unsigned short)(*ptr - 'a' + 10);
				else if('A' <= *ptr && *ptr <= 'F')
					subindex = (subindex << 4) + (unsigned short)(*ptr - 'A' + 10);
				else
					return false;
			}
			if(*ptr != '\0')
				return false;
			*puiIndex = index;
			*pucSubIndex = subindex;
			return true;
					   }
					   return false;
				}
				return false;
}

#if 0
static bool Ascii2Index(const char* lpszAscii, unsigned short* pValue)
{
	Q_ASSERT(pValue);
	Q_ASSERT(lpszAscii);

	*pValue = 0;

	int n = 0;
	const char* ptr;

	for(ptr = lpszAscii; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
		}
		else
			return false;
	}
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*pValue = (*pValue << 4) + (unsigned short)(*ptr - (char)'0');
		else if((char)'a' <= *ptr && *ptr <= (char)'f')
			*pValue = (*pValue << 4) + (unsigned short)(*ptr - (char)'a' + (char)10);
		else if((char)'A' <= *ptr && *ptr <= (char)'F')
			*pValue = (*pValue << 4) + (unsigned short)(*ptr - (char)'A' + (char)10);
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 4)
		return false;
	else
		return true;
}
#endif

#if 0
static bool Ascii2Subindex(const char* lpszAscii, unsigned char* pValue)
{
	Q_ASSERT(pValue);
	Q_ASSERT(lpszAscii);

	*pValue = 0;

	int n = 0;
	const char* ptr;

	for(ptr = lpszAscii; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
		}
		else
			return false;
	}
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*pValue = (*pValue << 4) + (unsigned char)(*ptr - (char)'0');
		else if((char)'a' <= *ptr && *ptr <= (char)'f')
			*pValue = (*pValue << 4) + (unsigned char)(*ptr - (char)'a' + (char)10);
		else if((char)'A' <= *ptr && *ptr <= (char)'F')
			*pValue = (*pValue << 4) + (unsigned char)(*ptr - (char)'A' + (char)10);
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 2)
		return false;
	else
		return true;
}
#endif

static bool Ascii2Unsigned8(const char* lpszAscii, unsigned char* pValue)
{
	Q_ASSERT(pValue);
	Q_ASSERT(lpszAscii);

	*pValue = 0;

	int n = 0;
	const char* ptr;

	for(ptr = lpszAscii; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
				if((char)'0' <= *ptr && *ptr <= (char)'9')
					*pValue = (*pValue << 4) + (unsigned char)(*ptr - (char)'0');
				else if((char)'a' <= *ptr && *ptr <= (char)'f')
					*pValue = (*pValue << 4) + (unsigned char)(*ptr - (char)'a' + (char)10);
				else if((char)'A' <= *ptr && *ptr <= (char)'F')
					*pValue = (*pValue << 4) + (unsigned char)(*ptr - (char)'A' + (char)10);
				else
					return false;
			}
			for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
				;
			if(*ptr != (char)'\0')
				return false;
			if(n > 2)
				return false;
			else
				return true;
		}
		// octal number
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 2 && *pValue > 037)
				n = 4;
			if((char)'0' <= *ptr && *ptr <= (char)'7')
				*pValue = (*pValue << 3) + (unsigned char)(*ptr - (char)'0');
			else
				return false;
		}
		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 3)
			return false;
		else
			return true;
	}
	// decimal number
	if(*ptr == (char)'+')
		if(*++ptr == (char)'\0')
			return false;
	for(; *ptr == (char)'0'; ptr++)
		;
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		// overflow?
		if(n == 2 && *pValue > 25)
			n = 4;
		if(n == 2 && *pValue == 25 && *ptr > (char)'5')
			n = 4;
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*pValue = (*pValue * 10) + (unsigned char)(*ptr - (char)'0');
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 3)
		return false;
	else
		return true;
}

#if 0
static bool Ascii2Integer8(const char* lpszAscii, char* pValue)
{
	Q_ASSERT(pValue);
	Q_ASSERT(lpszAscii);

	*pValue = 0;

	int n = 0;
	const char* ptr;

	for(ptr = lpszAscii; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
				if((char)'0' <= *ptr && *ptr <= (char)'9')
					*pValue = (*pValue << 4) + (unsigned char)(*ptr - (char)'0');
				else if((char)'a' <= *ptr && *ptr <= (char)'f')
					*pValue = (*pValue << 4) + (unsigned char)(*ptr - (char)'a' + (char)10);
				else if((char)'A' <= *ptr && *ptr <= (char)'F')
					*pValue = (*pValue << 4) + (unsigned char)(*ptr - (char)'A' + (char)10);
				else
					return false;
			}
			for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
				;
			if(*ptr != (char)'\0')
				return false;
			if(n > 2)
				return false;
			else
				return true;
		}
		// octal number
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 2 && *pValue > 037)
				n = 4;
			if((char)'0' <= *ptr && *ptr <= (char)'7')
				*pValue = (*pValue << 3) + (unsigned char)(*ptr - (char)'0');
			else
				return false;
		}
		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 3)
			return false;
		else
			return true;
	}
	// negative decimal number
	if(*ptr == (char)'-') {
		if(*++ptr == (char)'\0')
			return false;
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 2 && *pValue > 12)
				n = 4;
			if(n == 2 && *pValue == 12 && *ptr > (char)'8')
				n = 4;
			if((char)'0' <= *ptr && *ptr <= (char)'9')
				*pValue = (*pValue * 10) + (unsigned char)(*ptr - (char)'0');
			else
				return false;
		}
		*pValue = *pValue * -1;

		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 3)
			return false;
		else
			return true;
	}
	// positive decimal number
	if(*ptr == (char)'+')
		if(*++ptr == (char)'\0')
			return false;
	for(; *ptr == (char)'0'; ptr++)
		;
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		// overflow?
		if(n == 2 && *pValue > 12)
			n = 4;
		if(n == 2 && *pValue == 12 && *ptr > (char)'7')
			n = 4;
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*pValue = (*pValue * 10) + (unsigned char)(*ptr - (char)'0');
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 3)
		return false;
	else
		return true;
}
#endif

static bool Ascii2Unsigned16(const char* lpszAscii, unsigned short* pValue)
{
	Q_ASSERT(pValue);
	Q_ASSERT(lpszAscii);

	*pValue = 0;

	int n = 0;
	const char* ptr;

	for(ptr = lpszAscii; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
				if((char)'0' <= *ptr && *ptr <= (char)'9')
					*pValue = (*pValue << 4) + (unsigned short)(*ptr - (char)'0');
				else if((char)'a' <= *ptr && *ptr <= (char)'f')
					*pValue = (*pValue << 4) + (unsigned short)(*ptr - (char)'a' + (char)10);
				else if((char)'A' <= *ptr && *ptr <= (char)'F')
					*pValue = (*pValue << 4) + (unsigned short)(*ptr - (char)'A' + (char)10);
				else
					return false;
			}
			for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
				;
			if(*ptr != (char)'\0')
				return false;
			if(n > 4)
				return false;
			else
				return true;
		}
		// octal number
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 5 && *pValue > 017777)
				n = 7;
			if((char)'0' <= *ptr && *ptr <= (char)'7')
				*pValue = (*pValue << 3) + (unsigned short)(*ptr - (char)'0');
			else
				return false;
		}
		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 6)
			return false;
		else
			return true;
	}
	// decimal number
	if(*ptr == (char)'+')
		if(*++ptr == (char)'\0')
			return false;
	for(; *ptr == (char)'0'; ptr++)
		;
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		// overflow?
		if(n == 4 && *pValue > 6553)
			n = 6;
		if(n == 4 && *pValue == 6553 && *ptr > (char)'5')
			n = 6;
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*pValue = (*pValue * 10) + (unsigned short)(*ptr - (char)'0');
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 5)
		return false;
	else
		return true;
}

#if 0
static bool Ascii2Integer16(const char* lpszAscii, short* pValue)
{
	Q_ASSERT(pValue);
	Q_ASSERT(lpszAscii);

	*pValue = 0;

	int n = 0;
	const char* ptr;

	for(ptr = lpszAscii; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
				if((char)'0' <= *ptr && *ptr <= (char)'9')
					*pValue = (*pValue << 4) + (unsigned short)(*ptr - (char)'0');
				else if((char)'a' <= *ptr && *ptr <= (char)'f')
					*pValue = (*pValue << 4) + (unsigned short)(*ptr - (char)'a' + (char)10);
				else if((char)'A' <= *ptr && *ptr <= (char)'F')
					*pValue = (*pValue << 4) + (unsigned short)(*ptr - (char)'A' + (char)10);
				else
					return false;
			}
			for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
				;
			if(*ptr != (char)'\0')
				return false;
			if(n > 4)
				return false;
			else
				return true;
		}
		// octal number
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 5 && *pValue > 017777)
				n = 7;
			if((char)'0' <= *ptr && *ptr <= (char)'7')
				*pValue = (*pValue << 3) + (unsigned short)(*ptr - (char)'0');
			else
				return false;
		}
		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 6)
			return false;
		else
			return true;
	}
	// negative decimal number
	if(*ptr == (char)'-') {
		if(*++ptr == (char)'\0')
			return false;
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 4 && *pValue > 3276)
				n = 6;
			if(n == 4 && *pValue == 3276 && *ptr > (char)'8')
				n = 6;
			if((char)'0' <= *ptr && *ptr <= (char)'9')
				*pValue = (*pValue * 10) + (unsigned short)(*ptr - (char)'0');
			else
				return false;
		}
		*pValue = *pValue * -1;

		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 5)
			return false;
		else
			return true;
	}
	// positive decimal number
	if(*ptr == (char)'+')
		if(*++ptr == (char)'\0')
			return false;
	for(; *ptr == (char)'0'; ptr++)
		;
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		// overflow?
		if(n == 4 && *pValue > 3276)
			n = 6;
		if(n == 4 && *pValue == 3276 && *ptr > (char)'7')
			n = 6;
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*pValue = (*pValue * 10) + (unsigned short)(*ptr - (char)'0');
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 5)
		return false;
	else
		return true;
}
#endif

static bool Ascii2Unsigned32(const char* lpszAscii, unsigned long* pValue)
{
	Q_ASSERT(pValue);
	Q_ASSERT(lpszAscii);

	*pValue = 0;

	int n = 0;
	const char* ptr;

	for(ptr = lpszAscii; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
				if((char)'0' <= *ptr && *ptr <= (char)'9')
					*pValue = (*pValue << 4) + (unsigned long)(*ptr - (char)'0');
				else if((char)'a' <= *ptr && *ptr <= (char)'f')
					*pValue = (*pValue << 4) + (unsigned long)(*ptr - (char)'a' + (char)10);
				else if((char)'A' <= *ptr && *ptr <= (char)'F')
					*pValue = (*pValue << 4) + (unsigned long)(*ptr - (char)'A' + (char)10);
				else
					return false;
			}
			for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
				;
			if(*ptr != (char)'\0')
				return false;
			if(n > 8)
				return false;
			else
				return true;
		}
		// octal number
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			if(n == 10 && *pValue > 03777777777UL)
				n = 12;
			if((char)'0' <= *ptr && *ptr <= (char)'7')
				*pValue = (*pValue << 3) + (unsigned long)(*ptr - (char)'0');
			else
				return false;
		}
		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 11)
			return false;
		else
			return true;
	}
	// decimal number
	if(*ptr == (char)'+')
		if(*++ptr == (char)'\0')
			return false;
	for(; *ptr == (char)'0'; ptr++)
		;
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		// overflow?
		if(n == 9 && *pValue > 429496729UL)
			n = 11;
		if(n == 9 && *pValue == 429496729UL && *ptr > (char)'5')
			n = 11;
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*pValue = (*pValue * 10UL) + (unsigned long)(*ptr - (char)'0');
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 10)
		return false;
	else
		return true;
}

#if 0
static bool Ascii2Integer32(const char* lpszAscii, long* pValue)
{
	Q_ASSERT(pValue);
	Q_ASSERT(lpszAscii);

	*pValue = 0;

	int n = 0;
	const char* ptr;

	for(ptr = lpszAscii; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr == (char)'\0')
		return false;
	if(*ptr == '0') {
		if(*++ptr == (char)'\0')
			return true;
		// hexadecimal number
		if(*ptr == (char)'x' || *ptr == (char)'X') {
			for(++ptr; *ptr == '0'; ptr++)
				;
			for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
				if((char)'0' <= *ptr && *ptr <= (char)'9')
					*pValue = (*pValue << 4) + (unsigned long)(*ptr - (char)'0');
				else if((char)'a' <= *ptr && *ptr <= (char)'f')
					*pValue = (*pValue << 4) + (unsigned long)(*ptr - (char)'a' + (char)10);
				else if((char)'A' <= *ptr && *ptr <= (char)'F')
					*pValue = (*pValue << 4) + (unsigned long)(*ptr - (char)'A' + (char)10);
				else
					return false;
			}
			for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
				;
			if(*ptr != (char)'\0')
				return false;
			if(n > 8)
				return false;
			else
				return true;
		}
		// octal number
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			if(n == 10 && *pValue > 03777777777L)
				n = 12;
			if((char)'0' <= *ptr && *ptr <= (char)'7')
				*pValue = (*pValue << 3) + (unsigned long)(*ptr - (char)'0');
			else
				return false;
		}
		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 11)
			return false;
		else
			return true;
	}
	// negative decimal number
	if(*ptr == (char)'-') {
		if(*++ptr == (char)'\0')
			return false;
		for(; *ptr == (char)'0'; ptr++)
			;
		for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
			// overflow?
			if(n == 9 && *pValue > 214748364L)
				n = 11;
			if(n == 9 && *pValue == 214748364L && *ptr > (char)'8')
				n = 11;
			if((char)'0' <= *ptr && *ptr <= (char)'9')
				*pValue = (*pValue * 10UL) + (unsigned long)(*ptr - (char)'0');
			else
				return false;
		}
		*pValue = *pValue * -1L;

		for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
			;
		if(*ptr != (char)'\0')
			return false;
		if(n > 10)
			return false;
		else
			return true;
	}
	// positive decimal number
	if(*ptr == (char)'+')
		if(*++ptr == (char)'\0')
			return false;
	for(; *ptr == (char)'0'; ptr++)
		;
	for(; *ptr != (char)'\0' && *ptr != (char)'\t' && *ptr != (char)' '; ptr++, n++) {
		// overflow?
		if(n == 9 && *pValue > 214748364L)
			n = 11;
		if(n == 9 && *pValue == 214748364L && *ptr > (char)'7')
			n = 11;
		if((char)'0' <= *ptr && *ptr <= (char)'9')
			*pValue = (*pValue * 10UL) + (unsigned long)(*ptr - (char)'0');
		else
			return false;
	}
	for(; *ptr == (char)'\t' || *ptr == (char)' '; ptr++)
		;
	if(*ptr != (char)'\0')
		return false;
	if(n > 10)
		return false;
	else
		return true;
}
#endif

#if 0
static char ReadTextValue(CStdioFile* fpFile, CString& strValue)
{
	Q_ASSERT(fpFile);
	strValue.Empty();

	int i = 0;
	char ch;

	if(fpFile->Read(&ch, 1) != 1)
		return EOF;
	if(ch == '\"') {
		do	{
			if(fpFile->Read(&ch, 1) != 1)
				return EOF;
			if(ch != '\"')
				strValue += (char)ch;
		}	while(ch != '\"');
		if(fpFile->Read(&ch, 1) != 1)
			return EOF;
		return ch;
	}
	else {
		while(ch != '\n' && ch != '\t' && ch != ',' && ch != ';') {
			if(fpFile->Read(&ch, 1) != 1)
				return EOF;
			if(ch != '\n' && ch != '\t' && ch != ',' && ch != ';')
				strValue += (char)ch;
		}
		return ch;
	}
}
#endif

//////////////////////////////////////////////////////////////////////////////
//	revision control

const char* CCANopenReader::version()
{
	return (const char*)_id;
}

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
