////// $URL: http://svn.uv-software.com/opensource/linux/CANopen/COPbrowser/trunc/dic/IniReader.cpp $
//
//	project   :  IniReader - Windows INI-Files
//	
//	purpose   :  IniReader - a simple interface to read INI-Files
//	
//	copyright :  (C) 2009, UV Software, Friedrichshafen
//
//	compiler  :  GCC - GNU C++ Compiler (Linux Kernel 2.6)
//
//	export    :  class CIniReader
//	
//	includes  :  IniReader.h, <includes>
//	
//	author    :  Uwe Vogt, UV Software
//	
//	e-mail    :  uwe.vogt@uv-software.de
//
//////////////////////////////////////////////////////////////////////////////
//	IniReader: A Forward-Reader for INI-Files (e.g. CANopen EDS-Files).

static const char _id[] = "$Id: IniReader.cpp 56 2009-11-05 18:48:27Z saturn $";

//////////////////////////////////////////////////////////////////////////////
//	include-files

#include "IniReader.h"

//////////////////////////////////////////////////////////////////////////////
//	C-variables

static int  nLine = -1;
static bool bSection = false;

//////////////////////////////////////////////////////////////////////////////
//	class CIniReader

CIniReader::CIniReader( const char* fileName )
{
	m_file.setName( fileName );
	// open the specified file
	if( m_file.open( QIODevice::ReadOnly ) )
		nLine = 0;
}

CIniReader::CIniReader()
{
}

CIniReader::~CIniReader()
{
	// file opened? close the file
	if( m_file.isOpen() )
		m_file.close();
}

int	CIniReader::getLineOfError()
{
	return nLine;
}

int CIniReader::moveToNextItem()
{
	// file not opened? return immediately!
	if(!m_file.isOpen())
		return CIniReader::None;
	// ...
	char buf[65536];
	QString strLine; 

	while(m_file.readLine(buf, 65536) >= 0)
	{
		strLine = QString::fromAscii(buf);
		nLine++;

		/* remove newline character(s) */
		int n = strLine.length(); if(n > 0 && strLine.at(n-1) == '\n') strLine.truncate(n-1);
		int m = strLine.length(); if(m > 0 && strLine.at(m-1) == '\r') strLine.truncate(m-1);
		/* the string without newline! */
		
		if(!strLine.isEmpty())
		{
			int nLength = strLine.length(), i;
			
			QChar ch = strLine.at(0);

			switch(ch.toAscii())
			{
			//////////////////////////////////////////////////////////////////
			// scan a comment line (do not return)
			case ';':
			case ' ':
			case '\t':
				for(i = 0; i < nLength && strLine.at(i) != ';'; i++)
					if(strLine.at(i) != ' ' && strLine.at(i) != '\t')
						return CIniReader::Error;
				break;
			//////////////////////////////////////////////////////////////////
			// scan a section (do not return)
			case '[':
				m_section = "";
				m_key = "";
				m_value = "";
				
				if(nLength < 2)
					return CIniReader::Error;
				if(!strLine.at(1).isLetterOrNumber() && strLine.at(1) != '_')
					return CIniReader::Error;
				for(i = 1; i < nLength && strLine.at(i) != ']'; i++ )
				{
					if(!strLine.at(i).isLetterOrNumber() && strLine.at(i) != '_')
						return CIniReader::Error;
					m_section.append(strLine.at(i));
				}
				if(strLine[i++] != ']')
					return CIniReader::Error;
				if(m_section == "_")
					return CIniReader::Error;

				for(; i < nLength && strLine.at(i) != ';'; i++)
					if(strLine.at(i) != ' ' && strLine.at(i) != '\t')
						return CIniReader::Error;

				bSection = true;
				break;
			//////////////////////////////////////////////////////////////////
			// scan an item (<key>=<value>)
			default:
				m_key = "";
				m_value = "";

				if(m_section.isEmpty())
					return CIniReader::Error;
				if(!strLine.at(0).isLetterOrNumber() && strLine.at(0) != '_')
					return CIniReader::Error;
				for(i = 0; i < nLength && strLine.at(i) != '='; i++ )
				{
					if(!strLine.at(i).isLetterOrNumber() && strLine.at(i) != '_')
						return CIniReader::Error;
					m_key.append(strLine.at(i));
				}
				if(strLine[i++] != '=')
					return CIniReader::Error;
				if(m_key == "_")
					return CIniReader::Error;

				for(; i < nLength && strLine.at(i) != ';'; i++ )
				{
					m_value.append(strLine.at(i));
				}
				m_value.stripWhiteSpace();
					
				if(bSection)
				{
					bSection = false;
					return CIniReader::Section;
				}
				else
				{
					bSection = false;
					return CIniReader::Item;
				}
			}
		}
	}
	bSection = false;
	return CIniReader::None;
}

//////////////////////////////////////////////////////////////////////////////
//	revision control

const char* CIniReader::version()
{
	return (const char*)_id;
}

//////////////////////////////////////////////////////////////////////////////
//	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
//	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
//	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
