/*	-- $Header$ --
 *
 *	projekt   :  CAN - Controller Area Network
 *
 *	purpose   :  CANopen-over-TCP/IP Client
 *
 *	copyright :  (c) 2009, UV Software, Friedrichshafen
 *
 *	compiler  :  GCC - GNU C Compiler (Linux Kernel 2.6)
 *
 *	export    :  (see header file)
 *
 *	includes  :  tcp_open.h
 *
 *	author    :  Uwe Vogt, UV Software, Friedrichshafen
 *
 *	e-mail    :  uwe.vogt@uv-software.de
 *
 *
 *	-----------  description  -----------------------------------------------
 *
 *	CANopen-over-TCP/IP Client
 *
 *
 *	-----------  history  ---------------------------------------------------
 *
 *	$Log$
 */

static char _id[] = "$Id: tcp_open.c 60 2009-11-07 21:47:27Z saturn $";


/*  -----------  includes  -------------------------------------------------
 */

#include "tcp_open.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "base64.h"

#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/ioctl.h>
#include <net/if.h>

/*  -----------  defines  --------------------------------------------------
 */

#define MAX_BUFFER						1024

/*  -----------  types  ----------------------------------------------------
 */


/*  -----------  prototypes  -----------------------------------------------
 */

static size_t readline(int fd, char *buf, size_t nbyte);

extern int tcp_eval_response(char* response);
extern int tcp_eval_sdo_response(char* response, long* abort_code);
extern int tcp_eval_sdo_unsigned8(char* response, BYTE* value, long* abort_code);
extern int tcp_eval_sdo_unsigned16(char* response, WORD* value, long* abort_code);
extern int tcp_eval_sdo_unsigned32(char* response, DWORD* value, long* abort_code);
extern int tcp_eval_sdo_integer8(char* response, CHAR* value, long* abort_code);
extern int tcp_eval_sdo_integer16(char* response, SHORT* value, long* abort_code);
extern int tcp_eval_sdo_integer32(char* response, LONG* value, long* abort_code);
extern int tcp_eval_sdo_domain(char* response, short* length, BYTE* data, short nbyte, long* abort_code);
extern int tcp_eval_sdo_string(char* response, short* length, char* data, short nbyte, long* abort_code);
extern int tcp_eval_can_message(char* response, WORD* cob_id, short* length, BYTE* data);
extern int tcp_eval_rtr_message(char* response, short* length, BYTE* data);

/*  -----------  variables  ------------------------------------------------
 */

static int top_client = (-1);
static long top_sequence = 1;
static char top_buffer[MAX_BUFFER];
static struct _top_settings top_settings = {1, 1, 255};

/*  -----------  functions  ------------------------------------------------
 */

int top_init(DWORD addr, WORD port, struct _top_settings *settings, BYTE baudrate)
{
	struct sockaddr_in addr_in;

	memset(&addr_in, 0, sizeof(addr_in));
	addr_in.sin_family = AF_INET;
	addr_in.sin_addr.s_addr = htonl((unsigned long)addr);
	addr_in.sin_port = htons((unsigned short)port);

	if((top_client = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		return CANERR_TCPIP;
	}
	if(connect(top_client, (struct sockaddr*)&addr_in, sizeof(addr_in)) < 0) {
		close(top_client);
		top_client = (-1);
		return CANERR_TCPIP;
	}
	if(settings) {
		top_settings.net = settings->net;
		top_settings.node = settings->node;
		top_settings.master = settings->master;
	}
	baudrate = baudrate;
	return COPERR_NOERROR;
}

int top_exit()
{
	if(top_client > -1) {
		close(top_client);
		top_client = (-1);
		top_settings.net = 1;
		top_settings.node = 1;
		top_settings.master = 255;
		return COPERR_NOERROR;
	}
	else
		return COPERR_NOTINIT;
}

int top_reset(BYTE baudrate)
{
	long request, response;

	snprintf(top_buffer, MAX_BUFFER, "[%li] init %u\r\n", top_sequence++, baudrate);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	//...
	return TCPERR_NoError;
}

int top_write(BYTE node_id, WORD index, BYTE subindex, SHORT length, BYTE *data)
{
	long request, response; int i, j;
	int result; long abort;

	snprintf(top_buffer, MAX_BUFFER, "[%li] %u write 0x%X 0x%X d ", top_sequence++, node_id, index, subindex);
	for(i = 0, j = strlen(top_buffer); i < length; i += 3, j += 4)
	{
		base64_encode(&data[i], ((i + 3) < length) ? 3 : (length - i), (unsigned char*)&top_buffer[j], ((j + 4) < (MAX_BUFFER - 1)) ? 4 : ((MAX_BUFFER - 1) - j));
		top_buffer[((j + 4) < (MAX_BUFFER - 1)) ? (j + 4) : (MAX_BUFFER - 1)] = '\0';
	}
	strncat(top_buffer, "\r\n", MAX_BUFFER-strlen(top_buffer));

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	if((result = tcp_eval_sdo_response(top_buffer, &abort)) != TCPERR_Aborted)
		return result;
	else
		return abort;
}

int top_read(BYTE node_id, WORD index, BYTE subindex, SHORT *length, BYTE *data, SHORT max)
{
	long request, response;
	int result; long abort;

	snprintf(top_buffer, MAX_BUFFER, "[%li] %u read 0x%X 0x%X d\r\n", top_sequence++, node_id, index, subindex);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	if((result = tcp_eval_sdo_domain(top_buffer, length, data, max, &abort)) != TCPERR_Aborted)
		return result;
	else
		return abort;
}

int top_timeout(WORD new_value, WORD *old_value)
{
	long request, response;

	snprintf(top_buffer, MAX_BUFFER, "[%li] set sdo_timeout %u\r\n", top_sequence++, new_value);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	if(old_value)	/* FIXME! */
	{
		*old_value = 0;
	}
	return tcp_eval_response(top_buffer);
}

int top_write_8bit(BYTE node_id, WORD index, BYTE subindex, BYTE value)
{
	long request, response;
	int result; long abort;

	snprintf(top_buffer, MAX_BUFFER, "[%li] %u write 0x%X 0x%X u8 0x%X\r\n", top_sequence++, node_id, index, subindex, value);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	if((result = tcp_eval_sdo_response(top_buffer, &abort)) != TCPERR_Aborted)
		return result;
	else
		return abort;
}

int top_read_8bit(BYTE node_id, WORD index, BYTE subindex, BYTE *value)
{
	long request, response;
	int result; long abort;

	snprintf(top_buffer, MAX_BUFFER, "[%li] %u read 0x%X 0x%X u8\r\n", top_sequence++, node_id, index, subindex);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	if((result = tcp_eval_sdo_unsigned8(top_buffer, value, &abort)) != TCPERR_Aborted)
		return result;
	else
		return abort;
}

int top_write_16bit(BYTE node_id, WORD index, BYTE subindex, WORD value)
{
	long request, response;
	int result; long abort;

	snprintf(top_buffer, MAX_BUFFER, "[%li] %u write 0x%X 0x%X u16 0x%X\r\n", top_sequence++, node_id, index, subindex, value);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	if((result = tcp_eval_sdo_response(top_buffer, &abort)) != TCPERR_Aborted)
		return result;
	else
		return abort;
}

int top_read_16bit(BYTE node_id, WORD index, BYTE subindex, WORD *value)
{
	long request, response;
	int result; long abort;

	snprintf(top_buffer, MAX_BUFFER, "[%li] %u read 0x%X 0x%X u16\r\n", top_sequence++, node_id, index, subindex);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	if((result = tcp_eval_sdo_unsigned16(top_buffer, value, &abort)) != TCPERR_Aborted)
		return result;
	else
		return abort;
}

int top_write_32bit(BYTE node_id, WORD index, BYTE subindex, DWORD value)
{
	long request, response;
	int result; long abort;

	snprintf(top_buffer, MAX_BUFFER, "[%li] %u write 0x%X 0x%X u32 0x%lX\r\n", top_sequence++, node_id, index, subindex, value);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	if((result = tcp_eval_sdo_response(top_buffer, &abort)) != TCPERR_Aborted)
		return result;
	else
		return abort;
}

int top_read_32bit(BYTE node_id, WORD index, BYTE subindex, DWORD *value)
{
	long request, response;
	int result; long abort;

	snprintf(top_buffer, MAX_BUFFER, "[%li] %u read 0x%X 0x%X u32\r\n", top_sequence++, node_id, index, subindex);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	if((result = tcp_eval_sdo_unsigned32(top_buffer, value, &abort)) != TCPERR_Aborted)
		return result;
	else
		return abort;
}

int top_start_remote_node(BYTE node_id)
{
	long request, response;

	snprintf(top_buffer, MAX_BUFFER, "[%li] %u start\r\n", top_sequence++, node_id);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	return tcp_eval_response(top_buffer);
}

int top_stop_remote_node(BYTE node_id)
{
	long request, response;

	snprintf(top_buffer, MAX_BUFFER, "[%li] %u stop\r\n", top_sequence++, node_id);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	return tcp_eval_response(top_buffer);
}

int top_enter_preoperational(BYTE node_id)
{
	long request, response;

	snprintf(top_buffer, MAX_BUFFER, "[%li] %u preoperational\r\n", top_sequence++, node_id);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	return tcp_eval_response(top_buffer);
}

int top_reset_node(BYTE node_id)
{
	long request, response;

	snprintf(top_buffer, MAX_BUFFER, "[%li] %u reset node\r\n", top_sequence++, node_id);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	return tcp_eval_response(top_buffer);
}

int top_reset_communication(BYTE node_id)
{
	long request, response;

	snprintf(top_buffer, MAX_BUFFER, "[%li] %u reset communication\r\n", top_sequence++, node_id);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	return tcp_eval_response(top_buffer);
}

int top_transmit(LONG cob_id, SHORT length, BYTE *data)
{
	long request, response;
	char temp[6]; short i;

	if(length && !data)
	{
		return COPERR_NULLPTR;
	}
	snprintf(top_buffer, MAX_BUFFER, "[%li] send 0x%03lX %i\r\n", top_sequence++, cob_id, length);
	for(i = 0; i < length; i++)
	{
		snprintf(temp, 6, " 0x%X", data[i]);
		strncat(top_buffer, temp, MAX_BUFFER-strlen(top_buffer));
	}
	strncat(top_buffer, "\r\n", MAX_BUFFER-strlen(top_buffer));

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	return tcp_eval_response(top_buffer);
}

int top_request(LONG cob_id, SHORT *length, BYTE *data)
{
	long request, response;

	if(length)
	{
		return COPERR_NULLPTR;
	}
	snprintf(top_buffer, MAX_BUFFER, "[%li] send 0x%03lX rtr %i\r\n", top_sequence++, cob_id, *length);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	return tcp_eval_rtr_message(top_buffer, length, data);
}

int top_status(BYTE *status, BYTE *load)
{
	/*@ToDo: New command 'read status'? */
	if(status)
		*status = *status;
	if(load)
		*load = *load;
	return COPERR_NOTSUPP;
}

int top_queue_read(LONG *cob_id, SHORT *length, BYTE *data)
{
	long request, response;

	snprintf(top_buffer, MAX_BUFFER, "[%li] receive\r\n", top_sequence++);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	return tcp_eval_can_message(top_buffer, (WORD*)cob_id, length, data);
}

int top_queue_clear(void)
{
	/*§Note: Service not supported! */
	return COPERR_NOTSUPP;
}

int top_queue_status(BYTE *status, BYTE *load)
{
	/*@ToDo: New command 'read queue status'? */
	if(status)
		*status = *status;
	if(load)
		*load = *load;
	return COPERR_NOTSUPP;
}

int top_hardware(LPSTR version, short size)
{
	long request, response; short length;
	int result; long abort;

	snprintf(top_buffer, MAX_BUFFER, "[%li] info hardware\r\n", top_sequence++);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	if((result = tcp_eval_sdo_string(top_buffer, &length, version, size, &abort)) != TCPERR_Aborted)
		return result;
	else
		return abort;
}

int top_software(LPSTR version, short size)
{
	long request, response; short length;
	int result; long abort;

	snprintf(top_buffer, MAX_BUFFER, "[%li] info software\r\n", top_sequence++);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	if((result = tcp_eval_sdo_string(top_buffer, &length, version, size, &abort)) != TCPERR_Aborted)
		return result;
	else
		return abort;
}

int top_copyright(LPSTR string, short size)
{
	long request, response; short length;
	int result; long abort;

	snprintf(top_buffer, MAX_BUFFER, "[%li] info copyright\r\n", top_sequence++);

	if(send(top_client, top_buffer, strlen(top_buffer), 0) < 0)
	{
		return TCPERR_send;
	}
	if(sscanf(top_buffer, "[%li]", &request) != 1)
		request = 0;
	do
	{
		if(readline(top_client, top_buffer, MAX_BUFFER) > 0)
		{
			if(sscanf(top_buffer, "[%li]", &response) != 1)
				response = 0;
		}
		else
			break;
	}
	while (request != response);

	if((result = tcp_eval_sdo_string(top_buffer, &length, string, size, &abort)) != TCPERR_Aborted)
		return result;
	else
		return abort;
}

/*  -----------  local functions  ------------------------------------------
 */

static size_t readline(int fd, char *buf, size_t nbyte)
{
	char chr; size_t pos = 0, res;
	fd_set rdfs;
	struct timeval timeo;

    FD_ZERO(&rdfs);
    FD_SET(fd, &rdfs);

    timeo.tv_sec  = 66;
    timeo.tv_usec = 0;

	for(;;) {
		if(select(fd+1, &rdfs, NULL, NULL, &timeo) <= 0) {
    		return 0;
		}
		if(FD_ISSET(fd, &rdfs)) {
			if((res = recv(fd, &chr, sizeof(chr), 0)) > 0) {
				if(pos + 1 < nbyte) {
					buf[pos++] = chr;
					buf[pos] = 0;
				}
				if(chr == '\n') {
					return pos;
				}
			}
			else if(res == 0)
				return pos;
			else
				return res;
			//if(!running)
			//	return 0;
		}
	}
}

/*  -----------  revision control  -----------------------------------------
 */

char* top_version()
{
	return (char*)_id;
}

/*	--------------------------------------------------------------------------
 *	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
 *	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
 *	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
 */
