/*	-- $Header$ --
 *
 *	projekt   :  CAN - Controller Area Network
 *
 *	purpose   :  CANopen-over-TCP/IP Client
 *
 *	copyright :  (c) 2009, UV Software, Friedrichshafen
 *
 *	compiler  :  GCC - GNU C Compiler (Linux Kernel 2.6)
 *
 *	export    :  <export>
 *
 *	includes  :  tcp_defs.h, windows.h or default.h
 *
 *	author    :  Uwe Vogt, UV Software, Friedrichshafen
 *
 *	e-mail    :  uwe.vogt@uv-software.de
 *
 *
 *	-----------  description  -----------------------------------------------
 *
 *	CANopen-over-TCP/IP Client
 *
 *
 *	-----------  history  ---------------------------------------------------
 *
 *	$Log$
 */

#ifndef __TCP_OPEN_H
#define __TCP_OPEN_H

#ifdef __cplusplus
extern "C" {
#endif

/*  -----------  includes  -------------------------------------------------
 */

#include "tcp_defs.h"					// CAN definitions and options
#ifndef _WIN32
#include "default.h"					// C extensions and useful stuff
#else
#include <windows.h>					// Master include file for Windows
#endif

/*  -----------  defines  --------------------------------------------------
 */

#ifndef _COP_DEFS
#define _COP_DEFS
/*	- - - - - -  Baud rate index (CANopen Master)  - - - - - - - - - - - - - -
 */
 #define COPBDR_1000			0		// Baud rate: 1000 kBit/s
 #define COPBDR_800				1		// Baud rate: 800 kBit/s
 #define COPBDR_500				2		// Baud rate: 500 kBit/s
 #define COPBDR_250				3		// Baud rate: 250 kBit/s
 #define COPBDR_125				4		// Baud rate: 125 kBit/s
 #define COPBDR_100				5		// Baud rate: 100 kBit/s
 #define COPBDR_50				6		// Baud rate: 50 kBit/s
 #define COPBDR_20				7		// Baud rate: 20 kBit/s
 #define COPBDR_10				8		// Baud rate: 10 kBit/s

/*	- - - - - -  Error Codes (CAN/CANopen Communication)   - - - - - - - - - -
 */
 #define COPERR_NOERROR			 0		// No error
 #define COPERR_BOFF			-1		// Busoff status
 #define COPERR_EWRN			-2 		// Error warning status
 #define COPERR_BERR			-3		// Bus error
 #define COPERR_OFFLINE			-9		// Not started
 #define COPERR_ONLINE			-8		// Already started
 #define COPERR_MSG_LST			-10		// Message lost (overrun)
 #define COPERR_LEC_STUFF		-11		// Stuff error
 #define COPERR_LEC_FORM		-12		// Form error
 #define COPERR_LEC_ACK			-13		// Acknowledge error
 #define COPERR_LEC_BIT1		-14		// Recessive bit error
 #define COPERR_LEC_BIT0		-15		// Dominant bit error
 #define COPERR_LEC_CRC			-16		// Checksum error
 #define COPERR_TX_BUSY			-20		// Transmitter busy
 #define COPERR_RX_EMPTY		-30		// Receiver empty
 #define COPERR_QUE_OVR			-40		// Queue overrun
 #define COPERR_TIMEOUT			-50		// Time-out error
 #define COPERR_LENGTH			-60		// Invalid length
 #define COPERR_FORMAT			-70		// Format error
 #define COPERR_STATUS			-80		// Illegal status
 #define COPERR_NODE_ID			-90		// Illegal node-id
 #define COPERR_BAUDRATE		-91		// Illegal baudrate
 #define COPERR_ILLPARA			-93		// Illegal parameter
 #define COPERR_NULLPTR			-94		// Null-pointer assignment
 #define COPERR_NOTINIT			-95		// Not initialized
 #define COPERR_YETINIT			-96		// Already initialized
 #define COPERR_NOTSUPP			-98		// Not supported
 #define COPERR_FATAL			-99		// Other errors
 #define COPERR_ABORTED			0x4		// Transfer aborted

/*	- - - - - -  SDO Abort Codes according to CiA DS-301 Version 4.02  - - - -
 */
 #define SDOERR_WRONG_TOGGLEBIT 		0x05030000L
 #define SDOERR_PROTOCOL_TIMEOUT		0x05040000L
 #define SDOERR_UNKNOWN_SPECIFIER		0x05040001L
 #define SDOERR_INVALID_BLK_SIZE 		0x05040002L
 #define SDOERR_INVALID_SEQ_NUM  		0x05040003L
 #define SDOERR_CRC_ERROR          		0x05040004L
 #define SDOERR_OUT_OF_MEMORY			0x05040005L
 #define SDOERR_OBJECT_ACCESS			0x06010000L
 #define SDOERR_WRITE_ONLY_OBJECT		0x06010001L
 #define SDOERR_READ_ONLY_OBJECT		0x06010002L
 #define SDOERR_OBJECT_NOT_EXISTS		0x06020000L
 #define SDOERR_OBJECT_NOT_MAPABLE		0x06040041L
 #define SDOERR_PDO_TOO_LONG    		0x06040042L
 #define SDOERR_INVALID_VALUE   		0x06040043L
 #define SDOERR_INCOMPATIBILITY 		0x06040047L
 #define SDOERR_HARDWARE_ACCESS 		0x06060000L
 #define SDOERR_UNKNOWN_DATATYPE		0x06070010L
 #define SDOERR_TYPE_LENGTH_TOO_HIGH	0x06070012L
 #define SDOERR_TYPE_LENGTH_TOO_LOW 	0x06070013L
 #define SDOERR_SUBINDEX_NOT_EXISTS		0x06090011L
 #define SDOERR_RANGE_EXCEEDED   		0x06090030L
 #define SDOERR_RANGE_OVERFLOW  		0x06090031L
 #define SDOERR_RANGE_UNDERFLOW 		0x06090032L
 #define SDOERR_RANGE_MAX_LESS_MIN		0x06090033L
 #define SDOERR_GENERAL_ERROR			0x08000000L
 #define SDOERR_SERVICE_ERROR			0x08000020L
 #define SDOERR_LOCAL_ERROR				0x08000021L
 #define SDOERR_DEVICE_ERROR   			0x08000022L
 #define SDOERR_DYNAMIC_DICTIONARY		0x08000023L

/*	- - - - - -  LSS Error codes according to CiA DSP-305 Version 1.0  - - - -
 */
 #define LSSERR_ILLEGAL_NODE_ID			0x01
 #define LSSERR_BIT_TIMING_ERROR		0x01
 #define LSSERR_STORE_NOT_SUPPORTED		0x01
 #define LSSERR_STORAGE_ACCESS_ERROR	0x02
 #define LSSERR_IMPLEMENTATION_ERROR	0xFF
#endif
/*	- - - - - -  Error Codes (TCP/IP Communication)  - - - - - - - - - - - - -
 */
#ifndef  OK
 #define OK						 0		// Positives Ergebnis, heißt kein Fehler!
#endif
#define  TCPERR_NoError			 OK		// No error occurred
#define  TCPERR_NotSupported	-700	// Request not supported
#define  TCPERR_SyntaxError		-701	// Syntax error
#define  TCPERR_NotExceuted		-702	// Request not executed
#define  TCPERR_Timeout			-703	// Time-out occurred
#define  TCPERR_Winsock			-790	// socket: DLL error
#define  TCPERR_socket			-791	// socket: open socket
#define  TCPERR_connect			-792	// socket: connect
#define  TCPERR_send			-793	// socket: send data
#define  TCPERR_recv			-794	// socket: receive data
#define  TCPERR_close			-795	// socket: close socket
#define  TCPERR_Fatal			-999	// Fatal error
#define  TCPERR_Aborted			 4		// Transfer aborted


/*  -----------  types  ----------------------------------------------------
 */


/*  -----------  variables  ------------------------------------------------
 */


/*  -----------  prototypes  -----------------------------------------------
 */

int top_init(DWORD addr, WORD port, struct _top_settings *settings, BYTE baudrate);
int top_exit();

int top_reset(BYTE baudrate);

int top_write(BYTE node_id, WORD index, BYTE subindex, SHORT length, BYTE *data);
int top_read(BYTE node_id, WORD index, BYTE subindex, SHORT *length, BYTE *data, SHORT max);
int top_timeout(WORD new_value, WORD *old_value);
int top_write_8bit(BYTE node_id, WORD index, BYTE subindex, BYTE value);
int top_read_8bit(BYTE node_id, WORD index, BYTE subindex, BYTE *value);
int top_write_16bit(BYTE node_id, WORD index, BYTE subindex, WORD value);
int top_read_16bit(BYTE node_id, WORD index, BYTE subindex, WORD *value);
int top_write_32bit(BYTE node_id, WORD index, BYTE subindex, DWORD value);
int top_read_32bit(BYTE node_id, WORD index, BYTE subindex, DWORD *value);

int top_start_remote_node(BYTE node_id);
int top_stop_remote_node(BYTE node_id);
int top_enter_preoperational(BYTE node_id);
int top_reset_node(BYTE node_id);
int top_reset_communication(BYTE node_id);

int top_transmit(LONG cob_id, SHORT length, BYTE *data);
int top_request(LONG cob_id, SHORT *length, BYTE *data);
int top_status(BYTE *status, BYTE *load);

int top_queue_read(LONG *cob_id, SHORT *length, BYTE *data);
int top_queue_clear(void);
int top_queue_status(BYTE *status, BYTE *load);

int top_hardware(LPSTR version, short size);
int top_software(LPSTR version, short size);
int top_copyright(LPSTR string, short size);

char* top_version();
/*
 *	function  :  retrieve RCS info of this module as a string.
 *
 *	parameter :  (none)
 *
 *	result    :  pointer to RCS info (zero-terminated string)
 */

#ifdef __cplusplus
}
#endif
#endif /*__TCP_OPEN_H */

/*	--------------------------------------------------------------------------
 *	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
 *	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
 *	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
 */
